import java.util.*;
import java.io.*;

import javax.sound.midi.*;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;


class Interpret{
        
        static final int MAXMAPS=5;
		
	// constant indexes into statevariables[]
	static final int 
		XVAR=0,YVAR=1,ZVAR=2,
	        FXVAR=3,FYVAR=4,FZVAR=5,
	        UXVAR=6,UYVAR=7,UZVAR=8,
	        LXVAR=9,LYVAR=10,LZVAR=11,
	        LENVAR=12,DLENVAR=13, THICKVAR=14;
			 
	static StateVariable statevariables[]={
		new StateVariable(XVAR, "x","x"),
		new StateVariable(YVAR, "y","y"),
		new StateVariable(ZVAR, "z","z"),
		new StateVariable(FXVAR,"forwardx","Forward x"),
	        new StateVariable(FYVAR,"forwardy","Forward y"),
	        new StateVariable(FZVAR,"forwardz","Forward z"),
	        new StateVariable(UXVAR,"upx","Up x"),
	        new StateVariable(UYVAR,"upy","Up y"),
	        new StateVariable(UZVAR,"upz","Up z"),
	        new StateVariable(LXVAR,"leftx","Left x"),
	        new StateVariable(LYVAR,"lefty","Left y"),
		new StateVariable(LZVAR,"leftz","Left z"),
		new StateVariable(LENVAR,"length","Length"),
		new StateVariable(DLENVAR,"drawlength","Draw Length"),
		new StateVariable(THICKVAR,"thickness","Thickness")
	};
	static int numStateVariables=statevariables.length;
	
	Map[] maps=new Map[MAXMAPS];
	
	TurtleStack turtlestack;
	LStack timestack;
	// unimplemented stacks
	// LStack transposestack,dfactstack,vfactstack;
		
	int maxTrack=17;
	int maxChannel=16;
	int maxColor=256;
	int pLen;
        int curIdx,previdx;
        char curChar;
        int curMapNumber;
        LState turtle;

        DrawCanvas canvas;
        Sequence sequence;
        TrackInfo[] trackInfo= new TrackInfo[maxTrack];
        
        String production;
        boolean draw_p=true;
        double deltaAngleRad;
        double deltaAngleDeg;
        double length=0,curdrawlength;
        double deflscale = .9;   /* default scaling for '*/
        double defthickscale=.7;
        double lscale;
        boolean interruptMe=false;
        
        boolean normNewStateVariables=false;
        int tPQ;
        int track;

        double maxdrawingwide;
	long maxcumtime;
        public Interpret(){
        	for(int m=0;m<MAXMAPS;m++){
        		maps[m]=new Map(statevariables[XVAR],statevariables[DLENVAR],statevariables[THICKVAR]);
        		// default pitch, duration, and volume variables
        	}
        	for(int v=0;v<numStateVariables;v++){
        		// initialize the turtle's calculator to 
        		// recognize 'x','y','z','forwardx', etc
        		LState.calculator.getValu(statevariables[v].getToken()+"=0");
        			
        	}
        	
        	
        }
        void getNextChar(){
        	curChar=production.charAt(curIdx);
        	curIdx++;
        }
        int do_interpret(){
        	return do_interpret(production, true,deltaAngleDeg);
        }
        int do_interpret(String p, boolean mode,double angle){
                interruptMe=false;
                turtlestack= new TurtleStack();
	        if(mode==false){// if we are running the 'get limits' mode
	        	zeroStateVariablesMinMax();
			LState.newSeed();        	        
	        }
	        if(mode==true){ // running the 'make events' mode
	               // we need access to the sequence tracks 
        	       Track[] tracks=sequence.getTracks();
        	       for(int t=0;t<maxTrack-1;t++){
        	       		trackInfo[t]=new TrackInfo();
        	       		trackInfo[t].track=tracks[t+1];//+1 to make room for tempo track
        	       		trackInfo[t].channel=(char)(t %maxChannel);
/*
tracks[] 	0	1	2 ...	9	10 ...	16
chan  	     (tempo)	0	1 ...	8	9  ...	15
*/        	       		
        	       }
        	       tPQ=sequence.getResolution();

	        }
                timestack=new LStack();
                
		// some unimplemented optional stacks
                //transposestack=new LStack();
		//dfactstack=new LStack();
		//vfactstack=new LStack();

        	turtle=new LState();
        	turtle.resetSeed(); // we need the same seed set for both modes
        	
        	production=p;
        	pLen=production.length();
        	deltaAngleDeg=angle;
        	deltaAngleRad=LTools.deg2Rad(angle);
        	
        	// the turtle's starting state
        	turtle.orientAndPlace(deltaAngleRad,Vector3.X,Vector3.Y,Vector3.Z,0,0,0);
        	// the initial map
        	curMapNumber=0;
        	turtle.mymap=maps[curMapNumber];
        	
        	// the turtle informs the interpreter of its state
        	turtle.stateToVariables();
        	
        	curIdx=0;
        	double dang;
        	        	
        	double curval;
        	
        	double x,y,z;
        	
        	int numobjects=0;
        	boolean firstdraw=true;
        	while(curIdx<pLen){
        	        Thread.yield();
        	        if(interruptMe){
        	        	break;
        	        }
        	        x = turtle.x; y = turtle.y; z = turtle.z;
        	        
        	        previdx=curIdx;
        	        boolean holdingValue=false;
        	        curval=getNumeric(); // if there is a number here, just pick it up
        	        if(curIdx>previdx){
        	           holdingValue=true;
        	           //System.out.print("("+curval+")");
        	        }
        	        
        	        if(curIdx<pLen){
        		    previdx=curIdx;
        		    getNextChar();


        		    char thischar;
        		    switch(curChar){
        		        case 'F'://draw one length
                                case 'f':
                                case 'g'://move w/o drawing
                		case 'Z'://draw half length
                                case 'z':	
                                        thischar = curChar;
                                        
                			length=getDouble(turtle.length);
                			if(thischar=='Z' || thischar=='z'){
                        			if(curIdx > previdx){
                                                    length = 2*length;
                        			}
                        			curdrawlength=length/2;
                			}
					else{
					        curdrawlength=length;
					}
					if(mode == false){
                                                if(thischar == 'F'|| thischar=='Z' ||(turtle.inpoly>0 && thischar != 'g')) {
                                                       numobjects++;
                                                }
                                                if(firstdraw){
                                                    // waited until we know where
                                                    // and how the drawing starts before
                                                    // initializing min and max
                                                    initvariablelimits();
                                                    firstdraw=false;
                                                }
                                        }
                			turtle.moveForward(curdrawlength);
                			turtle.drawlength=curdrawlength;
                			//System.out.println("("+turtle.x+","+turtle.y+","+turtle.z+")");
                                        turtle.stateToVariables();
                                        
                                        if(mode==true){
                                                if(thischar == 'F'|| thischar=='Z'  || (turtle.inpoly>0 && thischar != 'g')){
                                                     //insertpatchandnote(turtle.track);
                				     insertpatchandnote();	
                                                     if(draw_p){
                                                        drawline(x,y,z);
                                                        //drawline(x,y,z,turtle.x,turtle.y,turtle.z,turtle.color%128 );
                                                     }
                
                                                }
                                                else {
                                                    insertrest();
                                                }
                                        }
                                        break; 
                                        
                                        
                           	case '\'': // set length scale                                        
                			lscale=getDouble(deflscale);
                			turtle.scaleLength(lscale);
                                        break;
                                case '"': // inverse set length scale
                                        lscale=getDouble(1.0/deflscale);
					turtle.scaleLength(lscale);					
                                        break;
                                case '!': // set thickness scale
                                        lscale=getDouble(defthickscale);
                                        turtle.scaleThickness(lscale);
                                        break;
                                case '?': // set inverse thickness
                                        lscale=getDouble(1.0/defthickscale);
                                        turtle.scaleThickness(lscale);
                                        break;
                                                                         
                                case '$': 
                                        turtle.rollToHorizontal();                                         
                                        break;
                                case '|':
					turtle.turnAround180();
                                        break;
                                case '%':
					turtle.rollAround180();
                                        break;    
                                case '+':
                                        dang=getDouble(180.0*turtle.deltaangle/Math.PI);
                                        dang = LTools.deg2Rad(dang);
                                        turtle.pitch(dang);
                                        break;
                                case '-':
                                        dang=getDouble(180.0*turtle.deltaangle/Math.PI);                                
                                        dang = LTools.deg2Rad(dang);
                                        turtle.pitch(-dang);
                                        break; 
                                case '&':
					dang=getDouble(180.0*turtle.deltaangle/Math.PI);                                
                                        dang = LTools.deg2Rad(dang);
					turtle.turn(dang);                                
                                        break;
                                case '^':                           
                                        dang=getDouble(180.0*turtle.deltaangle/Math.PI);
                                        dang = LTools.deg2Rad(dang);
                                        turtle.turn(-dang);
                                        break;
                                case '<':
					dang=getDouble(180.0*turtle.deltaangle/Math.PI);
                                        dang = LTools.deg2Rad(dang);
                                        turtle.roll(dang);
                                        break;
                                case '>': 
					dang=getDouble(180.0*turtle.deltaangle/Math.PI);                                
                                        dang = LTools.deg2Rad(dang);
                                        turtle.roll(-dang);                                         
                                        break;  
                                case ':': // scale the angle
                                        dang = getDouble(deflscale);
                                        turtle.scaleAngle(dang);
                                        /*if(deltaanglerad>TWOPI)deltaanglerad -= TWOPI;*/
                                        break;                                                                    
                                                        
                                case ';': // inverse scale the angle
                                        dang = getDouble(1/deflscale);
                                        turtle.scaleAngle(dang);
                                        /*if(deltaanglerad>TWOPI)deltaanglerad -= TWOPI;*/
                                        break;                                                                    
                                case '[':// push the turtle onto the stack
                 		case '{':
                 			turtlestack.pushTurtle(turtle);
                 			if (curChar == '{'){						
                                             int defTrack=(turtle.track+1)%(maxTrack-1);
                                             
                                             if(defTrack==9) defTrack=(defTrack+1)%(maxTrack-1);/***/
                                             track=getInt(defTrack);
                                             track=(track)%(maxTrack-1);/***/
                                             
                                             turtle.setTrack(track);
                                             
                                             turtle.inpoly++;
                                        }
        				break;
        			case ']': // pop turtle, but not its time
                 		case '}': // pop the turtle
                 			long holdtime=turtle.cumtime;
                 			LState testturtle=turtlestack.popTurtle();
                 			if(testturtle==null){
                 			// test the turtle
                 			// in case the stack was empty
                 			 
                 			     break;
                 			}
                 			turtle=testturtle;
                 			if(curChar==']'){//keep the time from the pre-pop state
                                            turtle.cumtime=holdtime;
                                        }
                                        if (curChar == '}'){
                                        
                                        }

					break;
                                case '\\': // push the time
                                        
                			timestack.push(new Long(turtle.cumtime));
                			//only change track/channel if specifically requested
                                        track=getInt(turtle.track);
                                        
                                        
                                        track=(track)%(maxTrack-1);/***/
                                        turtle.setTrack(track);
                                        break;              
                                case '/': // pop the time
                                        Long Ltesttime=(Long)timestack.pop();
                                        if(Ltesttime==null){
                                        	break;
                                        }
                                        turtle.cumtime=Ltesttime.longValue();

                                        break;
                                case 'c': // change (increment) the color/instrument
                                        int clr=getInt((turtle.color + 1) % (maxColor));
                                        turtle.setColor(clr);
                                        break;
                		case '*': // set MIDI track
                		        int defTrack=(turtle.track+1)%(maxTrack-1);                                             
                                        if(defTrack==9) defTrack=(defTrack+1)%(maxTrack-1);/***/
                                        track=getInt(defTrack);
                                        track=(track)%(maxTrack-1);/***/
                		        //track=getInt(turtle.track)%(maxTrack-1);
                		        turtle.setTrack(track);
                		        
                                        break;
                                case '~': 
                                        double maxrandomturn=getDouble(360);
                                        maxrandomturn = LTools.deg2Rad(maxrandomturn);                                        	
                                        turtle.randomTurn(maxrandomturn);
                                        
                                        break;
                                case 'm'://should the default (no arg) go to next map?
                                         
                                        int mapnumber=getInt(curMapNumber+1)%MAXMAPS;
                                        curMapNumber=mapnumber;
                                        turtle.setMap(maps[mapnumber]);
                                        break;
                                case 't': 
                                        turtle.transpose=getInt(0);
                                        break;                          
                                case 'd':
                                        double dfact = getDouble(1.0);                                    
                                        turtle.dfact=dfact;
                                        break;                                                      
                                case 'v':
                                        double vfact = getDouble(1.0);
                                        turtle.vfact=vfact;
                                        break;                                         

                                default:
        		    	        //System.out.print(curChar);
        		    	        
        		    		break;
        		    
        		    
        		    }
        		    turtle.stateToVariables();
        		    if(mode==false){
                               getvariablelimits();
                             }
                             else{
                               if(turtle.cumtime>maxcumtime){
                     			maxcumtime=turtle.cumtime;
                     		}
                     	     }
        		    
        		}
        		
        	}

        	if(mode==false){
        		setdrawingvariablewidths();
        		
        	}
        	else{ 
        	     
        	    //System.out.println("maxcumtime= "+maxcumtime);
        	}
        	return numobjects;
        }
        
        String getLimitsString(){
        // just make a string which tells the min and max of the
        // state variables
                String limitsString="";
                for(int i=0;i<numStateVariables;i++){
                  limitsString += statevariables[i].limitsToString();
		  limitsString+="\n";
		}
		
		return limitsString;

        }
        double getNumeric(){
                 if(curIdx>=pLen)
                 	return 0.0;
                 int prevCurIdx=curIdx;
                 StringBuffer NBuf=new StringBuffer("");
                 
                 // here is a kludge so that we can use
                 // decimal points as L-system symbols
                 char c=production.charAt(curIdx);
                 if(c=='.'){
                 	if(curIdx==pLen-1) return 0;
                 	else{   // peek at the next character
                 		c=production.charAt(curIdx+1);
                 		// if its not a digit, don't continue
                 		if(!Character.isDigit(c)){
                 		     return 0;
                 		}
                 	
                 	}
                 }// end kludge
                 double retval=0;
                 curIdx=LTools.getNumericChars(production,curIdx,NBuf);
                 // getNumericChars returns (p) the location in the string where it found the first
                 // character it doesn't consider to be part of a number.
         
                if(NBuf.length()>0){
                     
                     if(NBuf.charAt(0)=='('&&NBuf.charAt(NBuf.length()-1)==')'){
                     	// if the entire expression is surrounded by '()'
                     	// then remove them before parsing.
                     	int i=1;int level=1;
                     	
                     	while(i<NBuf.length()){
                     		char ch=NBuf.charAt(i);
                     		if(ch=='(') level++;
                     		if(ch==')') level--;
                     		
                     		i++;
                     		
                     		if(level==0) break;
                     	}
                     	if(i==NBuf.length()){
                     	  NBuf.deleteCharAt(0);
                     	  NBuf.deleteCharAt(NBuf.length()-1);
                     	}
                     }
                     retval=LTools.expressionParser.getValu(NBuf.toString());
                     
                } 
                return retval;
            
         }
         // getDouble(default) and getInt(default) return the number
         // at the current production string index if there is a 
         // number there, or return the default value if there is
         // no number there.
         double getDouble(double defVal){
                double myret=0;
         	previdx=curIdx;
         	double nret=getNumeric();
         	if(curIdx>previdx){
         		myret=nret;
         	}
         	else{
         		myret=defVal;
         	}
         	return myret;         	
         }
         int getInt(int defVal){
                int myret=0;
         	previdx=curIdx;
         	double nret=getNumeric();
         	if(curIdx>previdx){
         		myret=(int)nret;
         	}
         	else{
         		myret=defVal;
         	}
         	return myret;
         }
         void zeroStateVariablesMinMax(){
                for(int i=0;i<numStateVariables;i++){
	             statevariables[i].initMinMax(0);
		}
		for(int m=0;m<MAXMAPS;m++){
                     // specifically init the MinMax for pitch, duration
                     // and velocity variables which are 'new'
                     StateVariable pSV=maps[m].pitch;
                     StateVariable dSV=maps[m].duration;
                     StateVariable vSV=maps[m].volume;
                     if(pSV.turtleVariableIndex<0){
                           pSV.initMinMax(0);
                     }
                     if(dSV.turtleVariableIndex<0){
                     	   dSV.initMinMax(0);
                     }
                     if(vSV.turtleVariableIndex<0){
                     	   vSV.initMinMax(0);	
                     }
             
                }
         }              
         void variableLimits(){

                // First, make sure the turtle's state
                // is current
                turtle.stateToVariables();
                
                for(int i=0;i<numStateVariables;i++){
                     statevariables[i].initMinMax(turtle.variables[i]);
                }
                for(int m=0;m<MAXMAPS;m++){
                     // specifically init the MinMax for pitch, duration
                     // and velocity variables which are 'new'
                     StateVariable sV;
                     sV=maps[m].pitch;
                     if(sV.turtleVariableIndex<0)
                         sV.initMinMax(turtle.getStateVariableValue(sV));
                     sV=maps[m].duration;
                     if(sV.turtleVariableIndex<0)
                         sV.initMinMax(turtle.getStateVariableValue(sV));
                     sV=maps[m].volume;
                     if(sV.turtleVariableIndex<0)
                         sV.initMinMax(turtle.getStateVariableValue(sV));
                     
                }
                
         }
         void initvariablelimits(){
         // to get good min/max for the state variables
         // we initialize the mins and maxs to the values they
         // have at time the first L-object is made)
         	variableLimits();
         	maxcumtime=0;
         }

         void getvariablelimits(){
         // see if the turtle's state has new mins/maxs       
                for(int i=0;i<numStateVariables;i++){
                	statevariables[i].MinMax(turtle.variables[i]);
                }
                for(int m=0;m<MAXMAPS;m++){
                     // specifically set the MinMax for pitch, duration
                     // and velocity variables
                     StateVariable pSV=maps[m].pitch;
                     StateVariable dSV=maps[m].duration;
                     StateVariable vSV=maps[m].volume;
                     if(pSV.turtleVariableIndex<0){
                     	   
                           pSV.MinMax(turtle.getStateVariableValue(pSV));
                     }
                     if(dSV.turtleVariableIndex<0){

                           dSV.MinMax(turtle.getStateVariableValue(dSV));
                     }
                     if(vSV.turtleVariableIndex<0){

                           vSV.MinMax(turtle.getStateVariableValue(vSV));
                     }
                     
                }
                
                if(turtle.cumtime>maxcumtime)
                     maxcumtime=turtle.cumtime;
         } 
         void setdrawingvariablewidths(){
	   double xwide=statevariables[XVAR].getWidth(),
	          ywide=statevariables[YVAR].getWidth(),
	          zwide=statevariables[ZVAR].getWidth();
	   maxdrawingwide=(xwide>ywide)?xwide:ywide;
	   if(zwide>maxdrawingwide)maxdrawingwide=zwide;
         }       
         void drawline(double x1,double y1, double z1, double x2, double y2,double z2, int color){
         	// scale this line so that the whole thing 
         	// fits in a 1.0x1.0x1.0 cube
         	double sc=1/maxdrawingwide;
         	x1=sc*(x1-statevariables[XVAR].mymin);
         	y1=sc*(y1-statevariables[YVAR].mymin);
         	z1=sc*(z1-statevariables[ZVAR].mymin);
         	x2=sc*(x2-statevariables[XVAR].mymin);
         	y2=sc*(y2-statevariables[YVAR].mymin);
         	z2=sc*(z2-statevariables[ZVAR].mymin);
       		
         	canvas.drawline(x1,y1,z1, x2,y2,z2,Math.abs(color) );
         	
         }
         void drawline(double x1, double y1, double z1){
         	drawline(x1,y1,z1,turtle.x,turtle.y,turtle.z,turtle.color%128);
         }
         class TrackInfo{
         	Track track;
         	char channel;
                long cumtime=0;
        	long prevcumtime=0;
                int flags=0;
         }
         int getAPitch(){
                Map m=turtle.mymap;
         	StateVariable pvariable=m.pitch;
         	int pitchvarindex=pvariable.turtleVariableIndex;
         	double ppp;
		if(pitchvarindex<0){
		      
		      ppp=turtle.getStateVariableValue(pvariable);

		      if(normNewStateVariables){
		      	ppp= pvariable.normedFromMidRange(ppp);

		      }
		      //System.out.println(ppp+"p ");
		}
		else{
		      ppp=statevariables[pitchvarindex].normedFromMidRange(turtle);
		}
	        double pp=(m.pspread)*30*(ppp);
	        int p=m.scale.slideto(64+(int)pp,m.mode);
		p=Math.abs(m.transpose+turtle.transpose+p);// use abs value just to be sure
                return (int)(p)& 0x7F;
         }
         long getADuration(){
                Map m=turtle.mymap;
                StateVariable dvariable=m.duration;
                int durvarindex=dvariable.turtleVariableIndex;
                double ddd;
                if(durvarindex<0){
		      
		      ddd=turtle.getStateVariableValue(dvariable);
		      if(normNewStateVariables){
		      	ddd= dvariable.normedFromMidRange(ddd);
		      }
		      
		}
		else{
                      ddd=statevariables[durvarindex].normedFromMidRange(turtle);
                }
         	         	
         	double dd=(double)turtle.dfact*m.dmultiplier*tPQ*Math.pow(2,m.dspread*(ddd));
         	return (long)dd;
         }
         int getAVelocity(){
                Map m=turtle.mymap;
                StateVariable vvariable=m.volume;
                int velvarindex=vvariable.turtleVariableIndex;
		double vvv;
		if(velvarindex<0){
		        vvv=turtle.getStateVariableValue(vvariable);
		     	
		        if(normNewStateVariables){
		      	   vvv= vvariable.normedFromMidRange(vvv);
		        }
		}
		else{
         		vvv=statevariables[velvarindex].normedFromMidRange(turtle);
                }
         	double vv =64+ 30*m.vspread*vvv;
                int vel=(int)(vv*(turtle.vfact));
                
      		vel&=0x7F;
      		return (vel);
         }
         void insertrest(){
                long curtime=turtle.cumtime;
         	long dur=getADuration();
         	turtle.cumtime+=dur;
         }
         void insertpatch(){
                
                TrackInfo ti=trackInfo[turtle.track];
                Track curTrack=ti.track;
         	char curChannel=ti.channel;
         	long curtime=turtle.cumtime;
         	
         	int program =turtle.color%128;
         	while(program<0) program+=128;
         	MIDIEventStuff.createProgChangeEvent(curTrack,curChannel,program,curtime);
      		
               	
         }
         void insertnote(){
                
                TrackInfo ti=trackInfo[turtle.track];
                Track curTrack=ti.track;
         	char curChannel=ti.channel;
         	long curtime=turtle.cumtime;
         	long dur=getADuration();
         	
         	turtle.cumtime+=dur;
         	         	
                int pitch=getAPitch();      
                int vel=getAVelocity();
                MIDIEventStuff.createNoteOnAndOff(curTrack,(int)curChannel,pitch,vel,curtime,dur);
      		
               	
         }
         void insertpatchandnote(){
                
                TrackInfo ti=trackInfo[turtle.track];
                Track curTrack=ti.track;
         	char curChannel=ti.channel;
         	long curtime=turtle.cumtime;
         	long dur=getADuration();
         	int program =turtle.color%128;
         	//while(program<0) program+=128;
         	turtle.cumtime+=dur;

         	if(program>=0){         	
                        int pitch=getAPitch();      
                        int vel=getAVelocity();
                        
              		//System.out.print(" pitch"+pitch+" "+dur+" "+turtle.cumtime+"*");
              		MIDIEventStuff.createProgChangeEvent(curTrack,curChannel,program,curtime);              		
              		MIDIEventStuff.createNoteOnAndOff(curTrack,(int)curChannel,pitch,vel,curtime,dur);
              		
                }
               	
         }


}

class InterpretFrame extends JInternalFrame implements ActionListener, Runnable
{
        
        private Interpret interpreter;
        private DrawFrame drawframe; 
        private MIDIFrame midiframe;        
	
	
	private JButton interpretb,stopb;
	private JButton limitsb;
	private JCheckBox normNewStateVariablesCheck;
	
	private MapPane[] mappanels=new MapPane[Interpret.MAXMAPS];
	private DoubleInputField basicAngleInput,basicThicknessInput;
	
	private JComboBox newStateVariableBox;
	private int lastSVSelected=-1;
	private ProductionFrame productionframe=null; // 'point' to some production

	public InterpretFrame(){
	
		super("Interpretation",true,false,true,true);
		interpreter=new Interpret();
		constructFrame();
                                
	}
	public InterpretFrame(ProductionFrame pf, DrawFrame df,MIDIFrame mf){
	
		this();
                productionframe=pf;
                drawframe=df;
        	midiframe=mf;      
                interpreter.canvas=df.getCanvas();
                
	}

	void constructFrame(){
	
		// organize the frame components
		Container c = getContentPane();
                c.setLayout(new BorderLayout()); 
                
                JTabbedPane tabPane=new JTabbedPane();
                for(int t=0;t<Interpret.MAXMAPS;t++){
                     mappanels[t] =new MapPane(t);                
                     tabPane.addTab("Map "+Integer.toString(t),mappanels[t]) ;
                }
                
                JPanel panel1=new JPanel();
                interpretb=new JButton("Interpret");
                interpretb.setToolTipText("Interpret the production string.");
                panel1.add(interpretb);
                interpretb.addActionListener(this);
                
                stopb=new JButton("Stop");
                stopb.setToolTipText("Interrupt interpretation. Stop the madness!");
                panel1.add(stopb);
                stopb.setEnabled(false);
                stopb.addActionListener(this);
                
                basicAngleInput=new DoubleInputField("Angle",90,12);
                basicAngleInput.setToolTipText("Set the default turning angle"); 
                panel1.add(basicAngleInput); 
                basicThicknessInput=new DoubleInputField("Thickness",10,12); 
                basicThicknessInput.setToolTipText("Set the default thickness");
                panel1.add(basicThicknessInput);
                
                limitsb=new JButton("Limits");
                limitsb.setToolTipText("See state variable ranges");
                panel1.add(limitsb);
                limitsb.addActionListener(this);
                
                JPanel panel2=new JPanel();
                JPanel topPanel=new JPanel(new GridLayout(2,1));
                
                
               
                newStateVariableBox=new JComboBox();
                newStateVariableBox.setEditable(true);
                newStateVariableBox.addActionListener(this);
                               
                normNewStateVariablesCheck=new JCheckBox("Normed");
                normNewStateVariablesCheck.setToolTipText("Normalize all new state variables");
                panel2.add(new JLabel("New State Variable: "));
                panel2.add(newStateVariableBox);
                panel2.add(normNewStateVariablesCheck);
                
                topPanel.add(panel1);
                topPanel.add(panel2);
                c.add(tabPane,BorderLayout.CENTER);
                c.add(topPanel,BorderLayout.NORTH);  

                pack();
                
	
	}

	public void actionPerformed(ActionEvent e) {
	        Object esource=e.getSource();
	        
	 	if(esource==interpretb){
	 	     if(productionframe!=null){
	 	        Thread interpretthread=new Thread(this);
        	        interpretthread.start();
	 	      	
	 	     }
	 	}
	 	else if(esource==stopb){
	 		interpreter.interruptMe=true;
	 	}
	 	else if(esource==limitsb){
	 	        JOptionPane.showMessageDialog(null,interpreter.getLimitsString(),"State Variable Limits in Current Interpretation",JOptionPane.PLAIN_MESSAGE);
	 		
	 	}
	 	
	 	else if(esource==newStateVariableBox){
	 	        readNewStateVariableBox();
	 	        
	 	}
	 	
	 	
	 }
	 
	 public void run(){
	      // two runs ...
	    
	      // ...first one to get limits ...
	      interpreter.normNewStateVariables=normNewStateVariablesCheck.isSelected();
	      stopb.setEnabled(true);
	      drawframe.clear();
	      try{
	      // don't automatically bring up the drawframe	    
	            //drawframe.setIcon(false);/***/
	      
	            //drawframe.toFront();
	      
	            //drawframe.setSelected(true);	            
	      }catch(Exception e){e.printStackTrace();}
	      for(int i=0;i<interpreter.MAXMAPS;i++){
                     mappanels[i].fillMyMap();
              }
	      //Thread.yield();
	      int numobj=interpreter.do_interpret(productionframe.getProduction(),false,basicAngleInput.getValue());
	      
	      productionframe.fillDeclareBox();
	      if(interpreter.interruptMe){
	      	JOptionPane.showMessageDialog(null, "No events created yet");
	      	stopb.setEnabled(false);
	      	return;
	      }
	      
	      // ...second to make the piece
	      if(numobj>0){
	              
	              drawframe.toFront();
	              try{
	              	drawframe.setSelected(true);
	              }catch(Exception e){}
        	      try {
                            Sequence sequence = new Sequence(Sequence.PPQ, midiframe.ticksPerQ,interpreter.maxTrack);
                            interpreter.sequence=sequence;
                            Track[] tracks = sequence.getTracks();                
                            
                            MIDIEventStuff.createTempoEvent(midiframe.tempo,0,tracks[0]);
                            interpreter.do_interpret();
                            // show if variables were defined or changed in interpretation
                            productionframe.fillDeclareBox();
              		    if(interpreter.interruptMe){
              		        int iret=JOptionPane.showConfirmDialog(null,  "Use incomplete interpretation?","Interruption", JOptionPane.YES_NO_OPTION);
              		        if(iret!=0){stopb.setEnabled(false); return;}
              		    }              
                            MIDIEventStuff.setSequenceConductorLength(sequence);
                            midiframe.setNewSequence(sequence);
                           
        	 
                            
                      } catch (Exception ex) { ex.printStackTrace(); }  
	      }
	      else{
	      	JOptionPane.showMessageDialog(null, "No events created");
	      	
	      }
	      stopb.setEnabled(false);	              
              
	 }
	 void setBasicAngle(double tval){
	 	basicAngleInput.setValue(tval);
	 }
	 double getBasicAngle(){
	 	return basicAngleInput.getValue();
	 }

	 void setBasicThickness(double tval){
	 	basicThicknessInput.setValue(tval);
	 }
	 double getBasicThickness(){
	 	return basicThicknessInput.getValue();
	 }
	 void readNewStateVariableBox(){
	 	 	int sel=newStateVariableBox.getSelectedIndex();
	 	 	
	 	 	String str=(String)newStateVariableBox.getSelectedItem();
	 	 	
	 	        if (sel>=0)
	 	            lastSVSelected=sel;
		      	

	 		str=LTools.removeSpaces(str);
	 		str=str.toLowerCase();
	 		
	 	        int eqidx=str.indexOf('=');
	 	        if(eqidx!=0){ 
	 	                if(str.length()>0){ 
	 	                	java.awt.Toolkit.getDefaultToolkit().beep();
	 	                }
	 	        	return;
	 	        }
	 	        
	 	        String expr=str.substring(1,str.length());
			// expr is the formula for this variable
			if(expr.length()==0){ // There was just a lone '='
				if(lastSVSelected>=0){
				   newStateVariableBox.removeItemAt(lastSVSelected);
				   // remove the formula at this index from the lists
				   for(int i=0;i<interpreter.MAXMAPS;i++){
				
					MapPane mp=mappanels[i];
					mp.pitchPanel.stateVariableComboBox.removeItemAt(interpreter.numStateVariables+lastSVSelected);
					mp.durationPanel.stateVariableComboBox.removeItemAt(interpreter.numStateVariables+lastSVSelected);
					mp.velocityPanel.stateVariableComboBox.removeItemAt(interpreter.numStateVariables+lastSVSelected);
			
				   }  
				   newStateVariableBox.setSelectedIndex(-1);
				   return;
				}
			}
			 
			else{				
				int s=addNewStateVariable(expr);
				newStateVariableBox.setSelectedIndex(s);
				
				return;
			}
					      	
	 		newStateVariableBox.setSelectedIndex(lastSVSelected);
	 
	 }
	 int getNewStateVariableBoxIndex(String xpr){
	 	int numItems=newStateVariableBox.getItemCount();
	 	for(int i=0;i<numItems;i++){
	 	        String str="="+xpr;
			if(str.equals((String)newStateVariableBox.getItemAt(i))){
			        
				return i;
			}
		}
		return -1;
	 
	 }
	 int addNewStateVariable(String expr){
	                
	                int idx=getNewStateVariableBoxIndex(expr);
	                if(idx>=0){// expr is already in the box, so return its index
	                	return idx;
	                }
	                
			// must really be new
			for(int i=0;i<interpreter.MAXMAPS;i++){
				// add expr to the state variable lists in each map
				MapPane mp=mappanels[i];
				mp.pitchPanel.stateVariableComboBox.addItem(expr);
				mp.durationPanel.stateVariableComboBox.addItem(expr);
				mp.velocityPanel.stateVariableComboBox.addItem(expr);
			
			}
			newStateVariableBox.addItem("="+expr);
			return -1;
	 }
	 
	 void mapFileBufferedRead(BufferedReader in){
        	boolean keepGetting=true;
        	String inLine;
        	int mapnumber=0;
        	boolean nomap=false;
        	MapPane mp=mappanels[mapnumber];
                while(keepGetting){
	     
        	     try {
                        inLine = in.readLine();
                        if(inLine==null){
                            keepGetting=false;
                            break;
                        }
                        
                     } 
                     catch(IOException e) {
                           System.err.println("readLine() unsuccessful");
                           keepGetting=false;
                           break;
                     }
                     String inLineHold=new String(inLine);
                     
                     inLine=LTools.formatLineIn(inLine);
                     if(inLine.length()>0){
                     	int eqidx=inLine.indexOf('=');
                     	
                     	if(eqidx>0){
                     	
                     		String L=inLine.substring(0,eqidx);
                     		String R=inLine.substring(eqidx+1,inLine.length());
                     		String lL=L.toLowerCase();
                     		String lR=R.toLowerCase();
                     		int svIdx=stateVariableTokenToIndex(lR);
                     		double p1=0;
                     		if(svIdx<0){
                             		try{
                                           p1=Double.parseDouble(R);
                                        }
                                        catch(NumberFormatException e){
                                           // see if it is a number
                                           p1=LTools.expressionParser.getValu(R);
                                           
                                           /*???	
                     			   if (p1>=0&&((int)p1)<interpreter.numStateVariables){
                     			          svIdx=(int)p1;
                     			   }
                     			   */
                                        }  
                     		}
                     		if(lL.equals("file")){
                     		
                     			BufferedReader in2=null;

                                	try{
                                                in2=new BufferedReader(new FileReader(new File(lR))); 
                                        }catch(Exception e){System.err.println("could not open "+lR);}
                                	if(in2!=null){
                                		mapFileBufferedRead(in2);
                                
                                		try{
                                        		in2.close();
                                     		}catch(Exception e){}
                                     	}
                     		}
                     		if(lL.equals("statevariable")){
                     			addNewStateVariable(lR);
                     		}
                     		if(lL.equals("angle")){
                     			basicAngleInput.setValue(p1);
                     		}
                     		else if(lL.equals("tempo")){
                     			midiframe.setTempo((int)p1);	
                     		}
                     		else if(lL.equals("thickness")){
					basicThicknessInput.setValue(p1);
                     		}
                     		else if(lL.equals("normed")){
                     		        if(lR.equals("true")){
                     			    normNewStateVariablesCheck.setSelected(true);
                     			}
                     			else{
                     			    normNewStateVariablesCheck.setSelected(false);
                     			}
                     			
                     		}
                     		else if(lL.equals("mapnumber")){
                     			int m=(int) p1;
                     			if(m<interpreter.MAXMAPS)
					    mapnumber=m;
					mp=mappanels[mapnumber];
					nomap=false;
                     		}
                     		else if(lL.equals("pitch")){
                     		    JComboBox pCBox=mp.pitchPanel.stateVariableComboBox;
				    if(svIdx>=0){
                     			pCBox.setSelectedIndex(svIdx);
                     		    }
                     		    else{
                     		        
                     		        int idx=addNewStateVariable(lR);
                     		        mp.pitchPanel.setSelectedStateVariableToName(lR);
                     		        
                     		    }
                     		}
                     		else if(lL.equals("duration")){
                     		    JComboBox dCBox=mp.durationPanel.stateVariableComboBox;
				    if(svIdx>=0){
                     			dCBox.setSelectedIndex(svIdx);
                     		    }
                     		    else{

                     		        int idx=addNewStateVariable(lR);
                     		        mp.durationPanel.setSelectedStateVariableToName(lR);
                     		        
                     		    }
                     		
                     		}
                     		else if(lL.equals("volume")||lL.equals("velocity")){
				    JComboBox vCBox=mp.velocityPanel.stateVariableComboBox;
				    if(svIdx>=0){
                     			vCBox.setSelectedIndex(svIdx);
                     		    }
                     		    else{
                     		    
                     		        int idx=addNewStateVariable(lR);
                     		        mp.velocityPanel.setSelectedStateVariableToName(lR);                     		        
                     		        
                     		    }                     		
                     		}
                     		else if(lL.equals("pspread")){
                     		    if(p1!=0)
                     		       mp.pitchPanel.setSpreadValue(p1);                     		
                     		}
                     		else if(lL.equals("dspread")){
                     		    if(p1!=0)
                     		       mp.durationPanel.setSpreadValue(p1);                     		
                     		}
                     		else if(lL.equals("vspread")){
                     		    if(p1!=0)
                     		       mp.velocityPanel.setSpreadValue(p1);                     		
                     		}
                     		else if(lL.equals("transpose")){
                     			mp.transposeInput.setValue((int)p1);
                     		
                     		}
                     		else if(lL.equals("dmultiplier")||lL.equals("dmult")){
                     		      if(p1>=0)
                     			mp.setDMultSliderValue(p1);
                     		
                     		}
                     		else if(lL.equals("mode")){
                     			mp.modeInput.setValue((int)p1);
                     		
                     		}
				else if(lL.equals("scale")){
				     eqidx=inLineHold.indexOf('=');
                     	
                     		     if(eqidx>0){
                     	
                     		
                     			R=inLineHold.substring(eqidx+1,inLineHold.length());
                     			R=LTools.cleanUp(R);
					Scale sc=(Scale) Scale.scales.get(R);
                			if(sc!=null && !nomap){
                                                   
                                                   mp.scaleBox.setSelectedItem(sc.getName());
                                        }
                                        else {
                                                   Scale newscale=Scale.toScale(R);
                                                   if(newscale!=null){                                                       
                                                       for(int i=0;i<interpreter.MAXMAPS;i++){
                                                            mappanels[i].fillscaleBox();
                                                       }
                                                       if(!nomap){
                                                       	    mp.scaleBox.setSelectedItem(newscale.getName());
                                                       	    
                                                       }
                                                   } 
                                                   
                                        } 
                                     }                    
				}
				else{
				// assume everything else is a declaration expression
				   LTools.expressionParser.getValu(inLine);
				   productionframe.fillDeclareBox();
				}
                     		
                     	
                     	}
                     	else{                     	
                     	        String L=inLine.toLowerCase();
                     	        
                     	        if(L.equals("nomap")){
                     	        	nomap=true;
                     	        	
                     	        }
                     		
                     	}
                     	
                     }  
                       
                }
        		 	 	 	
	 }
	 
         boolean mapFileWrite(FileWriter fw, boolean includeComments){
                // just the actual writing
                boolean ret=true;
         	try{
         	     if(includeComments){
	 		fw.write("# LMUSe (Java version) Map file\r\n");
	 		fw.write("# Saved: "+new Date().toString()+"\r\n");
	 	     } 	 	     
	 	     fw.write("tempo="+midiframe.tempo+"\r\n");
	 	     fw.write("angle="+basicAngleInput.getValue()+"\r\n");
	 	     fw.write("thickness="+basicThicknessInput.getValue()+"\r\n");
	 	     fw.write("normed="+normNewStateVariablesCheck.isSelected()+"\r\n");
	 	     int numNewSV=newStateVariableBox.getItemCount();
	 	     for(int nsv=0;nsv<numNewSV;nsv++){
	 		        String expr=(String)newStateVariableBox.getItemAt(nsv);
	 		        int eqidx=expr.indexOf('=');
	 	                if(eqidx!=0) continue;	 	        
	 	        	expr=expr.substring(1,expr.length());      
	 	        	
	 		        fw.write("statevariable="+ expr +"\r\n");
		     }
		     // write 'new' scales
		     int icnt=mappanels[0].scaleBox.getItemCount();                                         
                     for(int i=0;i<icnt;i++){
                          	String scalename=(String)mappanels[0].scaleBox.getItemAt(i);
                          	if( Character.isDigit(scalename.charAt(0))){
                          		fw.write("scale="+ scalename +"\r\n");
                          	}
                     }
		     
	 	}catch(Exception e){return false;}
	 	for(int m=0;m<interpreter.MAXMAPS;m++){
	 	     MapPane mp=mappanels[m];
	 	     try{
	 		fw.write("\r\n");
	 		fw.write("mapnumber="+m+"\r\n");
	 		
	 		if( ! mp.writeMap(fw)){
	 			ret= false;
	 			System.out.println("writing map "+m+" failed");
	 			break;
	 		}
	 		
	 	     }catch(Exception e){ret=false; break;}
	 	}
	 	return ret;
         
         }

	 class MapPane extends JPanel implements ActionListener{
		oneVariablePanel pitchPanel   =new oneVariablePanel("Pitch");		
                oneVariablePanel durationPanel=new oneVariablePanel("Duration");
                oneVariablePanel velocityPanel=new oneVariablePanel("Velocity");
		JComboBox scaleBox=new JComboBox();
		IntInputField transposeInput;
		IntInputField modeInput;
		JSlider dmultslider;
                double dmultmin=0.0625;
		double dmultmax=8.0;
		Map myMap;
		public MapPane(int m){
		     super(new GridLayout(2,1));
		     
		     myMap=interpreter.maps[m];
		     JPanel vPanel=new JPanel();
		     pitchPanel.stateVariableComboBox.setSelectedIndex(myMap.pitch.turtleVariableIndex);
		     durationPanel.stateVariableComboBox.setSelectedIndex(myMap.duration.turtleVariableIndex);
		     velocityPanel.stateVariableComboBox.setSelectedIndex(myMap.volume.turtleVariableIndex);
		     
		     pitchPanel.setToolTipText("How pitches derive from the turtle's state");
		     durationPanel.setToolTipText("How durations derive from the turtle's state");
		     velocityPanel.setToolTipText("How note velocities derive from the turtle's state");
		     
		     vPanel.add(pitchPanel);
                     vPanel.add(durationPanel);
                     vPanel.add(velocityPanel);
                     add(vPanel);
                     
                     
                     JPanel panel1=new JPanel();
                     JPanel scalePanel=new JPanel();
                     scalePanel.add(scaleBox);
                     scaleBox.setToolTipText("Pitches are quantized to this scale");
                     scalePanel.setBorder(BorderFactory.createTitledBorder("Scale"));
                     fillscaleBox();
                     scaleBox.addActionListener(this);
                     scaleBox.setEditable(true);
                     
                     
                     modeInput=new IntInputField("Mode",0,2,true);
                     //modeInput.setToolTipText("'Tonic' scale step");
                     modeInput.setMinMax(0,12);                                         
                     scalePanel.add(modeInput);
                     transposeInput=new IntInputField("Transpose",0,2,true);
                     //transposeInput.setToolTipText("Transpose by half steps");
                     transposeInput.setMinMax(-24,24);
                    
                     scalePanel.add(transposeInput);
                     
                     panel1.add(scalePanel);
                     
                     JPanel dmultPanel=new JPanel();
                     dmultPanel.setBorder(BorderFactory.createTitledBorder("Duration multiplier"));
                     dmultPanel.setToolTipText("Multiply note durations");
		     dmultslider = new JSlider(JSlider.HORIZONTAL, 0, 127, 40);

		     int sv=(int)((1.0-dmultmin)*(127.0/(dmultmax-dmultmin)));
                     dmultslider.setValue(sv);
                     Hashtable labelTable = new Hashtable();
		     labelTable.put( new Integer( 1 ), new JLabel("1/16") );
	             labelTable.put( new Integer( sv ), new JLabel("1") );
	             labelTable.put( new Integer( sv+1*16 ), new JLabel("2") );
	             labelTable.put( new Integer( sv+3*16 ), new JLabel("4") );
		     labelTable.put( new Integer( sv+7*16 ), new JLabel("8") );
		     dmultslider.setLabelTable( labelTable );
                     dmultslider.setPaintTicks(true);
                     dmultslider.setPaintLabels( true );
                     dmultslider.setMajorTickSpacing(4);
                                            
                     dmultPanel.add(dmultslider);
                     panel1.add(dmultPanel);
                     


                     add(panel1);
		}
		double getDMultValue(){
		        double sv=(double)dmultslider.getValue()/127.0;
		        sv*=(dmultmax-dmultmin);
		        sv+=dmultmin;
		        return sv;
			
			
		}
		void setDMultSliderValue(double dmval){
			int sv=(int)((dmval-dmultmin)*(127.0/(dmultmax-dmultmin)));
                        dmultslider.setValue(sv);
		}
                public void actionPerformed(ActionEvent e) {
                   Object esource=e.getSource();

                   if(esource==scaleBox){                    		
                        String scalename = (String)scaleBox.getSelectedItem();                                
                        Scale curscale=(Scale) Scale.scales.get(scalename);
                                
			if(curscale!=null){
                                   myMap.scale=curscale;           
                        }
                        else {
                                   Scale newscale=Scale.toScale(scalename);
                                   if(newscale==null) return;
                                   myMap.scale=newscale; 
                                   for(int m=0;m<Interpret.MAXMAPS;m++){
                                            mappanels[m].fillscaleBox();
                                   }
                        }
                   }
		}
		void fillMyMap(){
			//scale should already be set
			String scalename = (String)scaleBox.getSelectedItem();                                
                        Scale curscale=(Scale) Scale.scales.get(scalename);
                        if(curscale!=null){
                                   myMap.scale=curscale;           
                        }
                        int pitchVarIdx=pitchPanel.getSelectedStateVariableIndex();
                        int durationVarIdx=durationPanel.getSelectedStateVariableIndex();
                        int velocityVarIdx=velocityPanel.getSelectedStateVariableIndex();
                        if(pitchVarIdx<interpreter.numStateVariables){
                        	myMap.pitch=interpreter.statevariables[pitchVarIdx];
                        }
                        else {
                        	String sVNm=pitchPanel.getSelectedStateVariableName();
                        	myMap.pitch=new StateVariable( -1, sVNm,sVNm,1);
  
                        }
                        if(durationVarIdx<interpreter.numStateVariables){
                        	myMap.duration=interpreter.statevariables[durationVarIdx];
                        }
                        else {
                        	String sVNm=durationPanel.getSelectedStateVariableName();
                        	myMap.duration=new StateVariable( -1, sVNm,sVNm,1);
  
                        }
                        
                        if(velocityVarIdx<interpreter.numStateVariables){
                        	myMap.volume=interpreter.statevariables[velocityVarIdx];
                        }
                        else {
                        	String sVNm=velocityPanel.getSelectedStateVariableName();
                        	myMap.volume=new StateVariable( -1, sVNm,sVNm,1);
  
                        }
                        
                        myMap.pspread=pitchPanel.getSpreadValue();
                        myMap.dspread=durationPanel.getSpreadValue();
                        myMap.vspread=velocityPanel.getSpreadValue();
                        myMap.dmultiplier=getDMultValue();
                        myMap.transpose=transposeInput.getValue();
                        myMap.mode=modeInput.getValue();
		}
		void fillscaleBox(){
                	//fill the scaleBox with items from the scale hashtable
                	                                

                	Iterator it=Scale.scales.keySet().iterator();

                	int icnt=scaleBox.getItemCount();
                	while(it.hasNext()){
                       	    String str=(String)it.next();

                       	    boolean fnd=false;
                       	    for(int i=0;i<icnt;i++){
                          	if(str.equals((String)scaleBox.getItemAt(i))){
                                    fnd=true;
                                }
                            }
                            if(fnd==false){
                                scaleBox.addItem(str); 
                            }
                     
                	}
                	scaleBox.setSelectedItem(myMap.scale.getName());
                	
                
        	}
        	boolean writeMap(FileWriter fw){
        	    try{
        	      
        	      int pVIdx	= pitchPanel.stateVariableComboBox.getSelectedIndex();
        	      int dVIdx = durationPanel.stateVariableComboBox.getSelectedIndex();
        	      int vVIdx = velocityPanel.stateVariableComboBox.getSelectedIndex();
        	      
        	      if(pVIdx<interpreter.numStateVariables){
		      	fw.write("pitch="+stateVariableIndexToToken(pVIdx)+"\r\n");
		      }
		      else{
		        fw.write("pitch="+pitchPanel.stateVariableComboBox.getSelectedItem()+"\r\n");
		      }
		      if(dVIdx<interpreter.numStateVariables){
		      	fw.write("duration="+stateVariableIndexToToken(dVIdx)+"\r\n");
		      }
		      else{
		        fw.write("duration="+durationPanel.stateVariableComboBox.getSelectedItem()+"\r\n");
		      }
		      if(vVIdx<interpreter.numStateVariables){
		      	fw.write("volume="+stateVariableIndexToToken(vVIdx)+"\r\n");
		      }
		      else{
		        fw.write("volume="+velocityPanel.stateVariableComboBox.getSelectedItem()+"\r\n");
		      }
		      
		      fw.write("pspread="+ pitchPanel.getSpreadValue()+"\r\n");
		      fw.write("dspread="+ durationPanel.getSpreadValue()+"\r\n"); 
		      fw.write("vspread="+ velocityPanel.getSpreadValue()+"\r\n");
		      
                      fw.write("scale="+ scaleBox.getSelectedItem()+"\r\n"); 
		      fw.write("transpose="+transposeInput.getValue()+"\r\n");	
		      fw.write("mode="+modeInput.getValue()+"\r\n");	
		      fw.write("dmultiplier="+getDMultValue()+"\r\n");

                    }catch(Exception e){return false;}
                    
                    return true;         	
        	}
        	
        	
	 }
	 String stateVariableIndexToToken(int svI){
        	if(svI<interpreter.numStateVariables)
        	   return interpreter.statevariables[svI].getToken();
        	return null; 
        	
         }
	 int stateVariableTokenToIndex(String str){
        	int nv=interpreter.numStateVariables;
        	for(int i=0;i<interpreter.numStateVariables;i++){
        		/*
        		if(str.equals(interpreter.statevariables[i].token)){
        		   return i;
        		   
        		}
        		*/
        		if(interpreter.statevariables[i].tokenEquals(str)){
        			return i;
        		}
        	}
        	return -1;        		
	 }
	 class oneVariablePanel extends JPanel{
	        double spreadmin=-4.0,spreadmax=4.0;
	        JSlider sb;
	        JComboBox stateVariableComboBox;
		public oneVariablePanel(String name){
			super(new GridLayout(4,1));
			String[] stateVariableStrings=new String[interpreter.numStateVariables];
			for(int i=0;i<interpreter.numStateVariables;i++){
				stateVariableStrings[i]=interpreter.statevariables[i].getName();
				
			}
			stateVariableComboBox=new JComboBox(stateVariableStrings);
			
			add(stateVariableComboBox);

                        setBorder(BorderFactory.createTitledBorder(name));

			add(new JLabel("Spread"));
			sb = new JSlider(JSlider.HORIZONTAL, 0, 80, 40);
			int sv=(int)((1.0-spreadmin)*(80.0/(spreadmax-spreadmin)));
                        sb.setValue(sv);
                        
                        Hashtable labelTable = new Hashtable();
			labelTable.put( new Integer( 0 ), new JLabel("-4") );
			labelTable.put( new Integer( 20 ), new JLabel("-2") );
			labelTable.put( new Integer( 40 ), new JLabel("0") );
			labelTable.put( new Integer( 60 ), new JLabel("+2") );
			labelTable.put( new Integer( 80 ), new JLabel("+4") );
			sb.setLabelTable( labelTable );

                        sb.setPaintTicks(true);
                        sb.setPaintLabels( true );
                        sb.setMajorTickSpacing(10);
                        sb.setMinorTickSpacing(2);                        
                        add(sb);
		}
		int getSelectedStateVariableIndex(){
			return stateVariableComboBox.getSelectedIndex();
		}
		String getSelectedStateVariableName(){
			return (String)stateVariableComboBox.getSelectedItem();
		}
		double getSpreadValue(){
		    int sbval=sb.getValue();
		    double dval=(double)(sbval)/80.0*(spreadmax-spreadmin);
		    return dval+spreadmin;
		    
		}
		void setSpreadValue(double val){
		       int sv=(int)((val-spreadmin)*(80.0/(spreadmax-spreadmin)));
                        sb.setValue(sv);
		}
		void setSelectedStateVariableToName(String name){
		        int numItems=stateVariableComboBox.getItemCount();
			for(int n=0;n<numItems;n++){
                        	String tok=(String)stateVariableComboBox.getItemAt(n);
                        	if(tok.equals(name)){
                        		stateVariableComboBox.setSelectedIndex(n);
                     			break;
                        	}
                     		        	
                        }
                }
	
	 }
}


