import java.util.*;
import java.io.*;

import javax.sound.midi.*;
import javax.swing.*;
import javax.swing.event.*;

import java.awt.*;
import java.awt.event.*;

class MIDIFrame extends JInternalFrame implements ActionListener,ChangeListener 
{
	
	Sequencer sequencer;
	int ticksPerQ=96;        
        
        int tempo=120;
        
        int durQType=0; // how to quantize durations
        int qAmountIdx=0; // index into the combobox array of quantization ticks
        
        boolean separateInstrumentTracks=false;
        
	private Synthesizer synth;
	private Sequence sequence,
	         lastsequence=null;
	private JButton playButton;
	private JButton quantizeButton;
	MIDICanvas midicanvas=null;
	MidiChannel[] channels;
	
        private int tempomin=10;
	private int tempomax=460;
	
	private JSlider temposlider;
	private JLabel  tempotext;
	private final int BPSliderSteps=5;
	private int numtemposlidersteps=(tempomax-tempomin)/BPSliderSteps;
	
	private JSlider seekslider;
	private boolean sillyflag=false;	
	long seqDur=0;

	private JSlider volumeslider;
	
	public MIDIFrame(String title){
		super(title,true,false,false,true);
		synth=null;
        	try {
            		if (synth == null) {
                		if ((synth = MidiSystem.getSynthesizer()) == null) {
                    			System.err.println("getSynthesizer() failed!");
                    			return;
                		}
                		channels = synth.getChannels();
            		}
            		synth.open();
            		sequencer = MidiSystem.getSequencer();

            		//sequencer.addMetaEventListener(this);
            		
                         
        	} catch (Exception ex) { ex.printStackTrace(); return; } 
        	


		//getContentPane().setLayout(new GridLayout(2,2));
		getContentPane().setLayout(new BorderLayout());

				                
                JPanel buttonPanel=new JPanel();
                playButton=new JButton("Play");                
                buttonPanel.add(playButton);
                playButton.addActionListener(this);
		
		quantizeButton=new JButton("q");
		buttonPanel.add(quantizeButton);
		quantizeButton.addActionListener(this);

                getContentPane().add(buttonPanel,BorderLayout.NORTH);
                
                midicanvas=new MIDICanvas();
               
                seekslider = new JSlider(JSlider.HORIZONTAL, 0, 100, 0);
                seekslider.addChangeListener(this);
               
                JPanel seekPanel=new JPanel(new BorderLayout());
                
                seekPanel.add(seekslider,BorderLayout.NORTH);
		seekPanel.add(midicanvas,BorderLayout.SOUTH);
		
		getContentPane().add(seekPanel,BorderLayout.CENTER);
	                
                JPanel tempoPanel=new JPanel(new GridLayout(2,1));
                tempoPanel.setBorder(BorderFactory.createTitledBorder("Tempo")); 
                
		temposlider = new JSlider(0, numtemposlidersteps);
		
		tempotext=new JLabel();
                int tv=setTempoSlider(tempo);
                temposlider.addChangeListener(this);
                Hashtable labelTable = new Hashtable();	        
		temposlider.setLabelTable( labelTable );
                tempoPanel.add(tempotext);                
                tempoPanel.add(temposlider);

		JPanel volumePanel=new JPanel();
                volumePanel.setBorder(BorderFactory.createTitledBorder("Volume")); 
		volumeslider = new JSlider(0, 100);
		volumeslider.setValue(75);
                volumeslider.addChangeListener(this);
                volumePanel.add(volumeslider);
                
                JPanel lowerPanel=new JPanel(new GridLayout(2,1));
                lowerPanel.add(tempoPanel);
                lowerPanel.add(volumePanel);
                getContentPane().add(lowerPanel,BorderLayout.SOUTH);
		//getContentPane().add(tempoPanel,BorderLayout.SOUTH);
		pack();
	}
	boolean setNewSequence(Sequence seq){
	
	        sequence=seq;	            
        	saveMidiFile(new File("last.mid"));
           
	        try{
	            setIcon(false);
	            toFront();
	            try{
	                setSelected(true);
	            }
	            catch(Exception e){}

	            
	            
	        }catch(Exception e){return false;}
	        midicanvas.setSequence(sequence);
	        if(sequencer.isRunning()==false){
	            	seekslider.setValue(0);
	        }
	        setTempo(tempo);
		return true;
	
	}
	public void stateChanged(ChangeEvent e) {
	// implementing ChangeListener (sliders)
		Object esource=e.getSource();
	        if(esource==temposlider){
	        	
	        	int oldtempo=tempo;
	        	
	        	tempo=getTempoSliderTempo();
	        	setTempo(tempo);
	        	   
	        }
	        else if(esource==seekslider){
	        
	                if(sequence!=null && !sillyflag){
	        		
	        		int val=seekslider.getValue();
	        		
				long pos= (long)(((double)val/100)*seqDur);
				sequencer.setMicrosecondPosition(pos);
				
			}
	        
	        }
	        else if(esource==volumeslider){
	        	setVolume();
	        }
	        
	}
	public void meta(MetaMessage message) {
            if (message.getType() == MIDIEventStuff.TEMPO) {  
                 setTempo((int) MIDIEventStuff.tempoMessageToTempo(message));
            }
        }
        void stopSequencer(){
        	sequencer.stop();
                playButton.setText("Play"); 
        }
        void startSequencer(){
        	        try {
                                sequencer.open();
                                sequencer.setSequence(sequence);
                                
                        } catch (Exception ex) { ex.printStackTrace(); return;}

                        playButton.setText("Stop");
                        
                        //MIDIEventStuff.setSequenceTempo(sequence,getTempoSliderTempo());/***/
                        //sequencer.setTempoInBPM(getTempoSliderTempo());
        	        if(sequence!=lastsequence){
				sequencer.setMicrosecondPosition(0);
				seekslider.setValue(0);
							
			}
			lastsequence=sequence;
			int tpo=MIDIEventStuff.getSequenceTempo(sequence);
				
			if(tpo>0)
				   setTempo(tpo);
			setTempo((int)(sequencer.getTempoInBPM()));
			setVolume();
				
                        sequencer.start();
                        	                        	                        	
                        int val=seekslider.getValue();
                        	
                        
                        double dur=sequencer.getMicrosecondLength();
                        seqDur=(long)dur;
			long pos= (long)(((double)val/100)*dur);
			sequencer.setMicrosecondPosition(pos);
                        
                        	
                        SequencerWatcher sw=new SequencerWatcher(sequencer);
        }
	public void actionPerformed(ActionEvent e) {
	        Object esource=e.getSource();
	        
	        if(esource==playButton){
                    if(sequencer.isRunning()){
                        stopSequencer();
                                                             
                    }
                    else{
                        if(sequence!=null){
                        	startSequencer();        
                        
                        }
                        
                    }
                }
                else if(esource==quantizeButton){
                	showQuantizeDialog();
                	
                }
                

 
	}
	public void setVolume() {
        	double value = volumeslider.getValue() / 100.0;
		
        
                for (int i = 0; i < channels.length; i++) {                
				channels[i].controlChange(7, (int)(value * 127.0));

		}
        }
    
	int setTempoSlider(int tempoBPM){
	       
	        int tV=(tempoBPM-tempomin)/BPSliderSteps;
		temposlider.setValue(tV);
		tempotext.setText(Integer.toString(tempoBPM));
		return tV;	
	}
	int getTempoSliderTempo(){
		        
	        	return (int)(temposlider.getValue()*BPSliderSteps)+tempomin;

	}
	boolean setTempo(int tempoBPM){
	         if(tempoBPM<tempomin||tempoBPM>tempomax){
	         	return false;
	         }  
	         tempo=tempoBPM;
	         setTempoSlider(tempo);
                 
                 if(sequencer!=null){

                        
                 	sequencer.setTempoInBPM(tempo);
                 	
              
                 }
                 
                 return true;
	} 
	
	
	
	Sequence instrumentsToTracks(){
		if(sequence==null) return null;
		Sequence newSequence = null;
        	try{
        		newSequence =new Sequence(Sequence.PPQ, ticksPerQ);
        	}catch (Exception ex) { ex.printStackTrace(); return null;} 
        	
        	MIDIEventStuff.instrumentsToTracks(sequence,newSequence);
        	return newSequence;
	}
	
	public void saveMidiFile(File file) {
	    if(sequence==null) return;
	    Sequence saveSequence=sequence;
	    
	    MIDIEventStuff.removeUnusedTracks(saveSequence);
	    
	    if(separateInstrumentTracks){
	    	saveSequence=instrumentsToTracks();
	    	if(saveSequence==null){
	    		saveSequence=sequence;
	    	}
	    }
	    else{
	    	MIDIEventStuff.removeExtraProgramChanges(saveSequence);
	    }
	    
            try {
                int[] fileTypes = MidiSystem.getMidiFileTypes(saveSequence);
                if (fileTypes.length == 0) {
                    System.out.println("Can't save sequence");
                } 
                else {
                    if (MidiSystem.write(saveSequence, fileTypes[0], file) == -1) {
                        throw new IOException("Problems writing MIDI file");
                    } 
                }
            } 
            catch (SecurityException ex) {} 
            catch (Exception ex) { 
                ex.printStackTrace(); 
            }
            
        }
        
        class SequencerWatcher implements Runnable{
        // at least for now, all this does is update the 'seek' slider
        // and check if the sequencer done
                Sequencer sequencer;
                boolean flag;
                Thread mythread;
        	SequencerWatcher(Sequencer seq){
        		sequencer=seq;
        		flag=true;
        		mythread=new Thread(this);
        		mythread.setPriority(Thread.MIN_PRIORITY);
        		mythread.start();
        		playButton.setText("Stop");
        	}
        	public void run(){
        	     while(flag==true){
        		if(!sequencer.isRunning()){
        			flag=false;
        			
        			playButton.setText("Play");
        			
        		}
			
			long pos=sequencer.getMicrosecondPosition();
			
			
			double frac=(double)pos/(double)seqDur;
			int val=(int)(frac*100);
			sillyflag=true;
						
			seekslider.setValue(val);			
						
			sillyflag=false;
        		//System.out.println(pos+" "+dur+" "+frac*100+" "+sequencer.getTempoFactor());
        		Thread.yield();
        		        		
        	     }
        	}
        
        }
      
 	void insertAllProgramChanges(){
 	        Sequence newSequence = null;
        	try{
        		newSequence =new Sequence(Sequence.PPQ, ticksPerQ);
        	}catch (Exception ex) { ex.printStackTrace(); return;} 
        	Track[] tracks=sequence.getTracks();
        	int numTracks=tracks.length;
        	for(int tr=0;tr<numTracks;tr++){
        		Track oldTrack=tracks[tr];
        		
        		Track newTrack=newSequence.createTrack();
        		MIDIEventStuff.insertProgramChanges(oldTrack,newTrack);
 		}
 		sequence=newSequence;
 		
 	}
        int showQuantizeDialog(){
        	
        	
        	JRadioButton[] rB = new JRadioButton[3];
        	ButtonGroup qGroup = new ButtonGroup();
        	rB[0] = new JRadioButton("Don't quantize durations");
                rB[1] = new JRadioButton("Quantize durations");
                rB[2] = new JRadioButton("Set note durations to quantize amount");
                rB[durQType].setSelected(true);

                String [] qAmounts={"1/64","1/48","1/32","1/24","1/16","1/12","1/8","1/6","1/4","1/2"};
                JComboBox qTicksBox=new JComboBox(qAmounts);
                qTicksBox.setSelectedIndex(qAmountIdx);
                Object[] componentArray=new Object[4];
                componentArray[0]=qTicksBox;
                for (int i = 0; i < 3; i++) {
                    qGroup.add(rB[i]);
                    componentArray[i+1]=rB[i];
                }

                int dret=JOptionPane.showOptionDialog(this, componentArray, "Quantize", JOptionPane.OK_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE, null, null, null);
                if(dret==0){
                	for (int i = 0; i < 3; i++) {
                    		if(rB[i].isSelected()){
                    			durQType=i;
                    		}
                	}
                	qAmountIdx=qTicksBox.getSelectedIndex();
                	int qAmount=48;
                	switch(qAmountIdx){
                		case 0:
                			qAmount=96/16;
                			break;
                		case 1:
                			qAmount=96/12;
                			break;
                		case 2:
                			qAmount=96/8;
                			break;
                	        case 3:
                			qAmount=96/6;
                			break;
                		case 4:
                			qAmount=96/4;
                			break;
                		case 5:
                			qAmount=96/3;
                			break;
                		case 6:
                			qAmount=96/2;
                			break;
                		case 7:
                			qAmount=2*96/3;
                			break;
                		case 8:
                			qAmount=96;
                			break;
                		case 9:
                			qAmount=96*2;
                			break;
                			

                	}
			Sequence newSequence=quantizeSequence(qAmount,durQType);
			if(newSequence!=null){
				setNewSequence(newSequence);
			
			}
                	return 1;
                }
                return 0;
                
        }
       
        Sequence quantizeSequence(int qTicks,int qType){
        
        	// returns a quantized version of the sequence
        	
        	Sequence newSequence = null;
        	try{
        		newSequence =new Sequence(Sequence.PPQ, ticksPerQ);
        	}catch (Exception ex) { ex.printStackTrace(); return null;} 
        	
        	MIDIEventStuff.quantizeSequence(sequence,newSequence,qTicks,qType);
        	return newSequence;
        	
        }

}




class MIDIEventStuff{
        static final int NOTEOFF           = 0x80;
        static final int NOTEON            = 0x90;
        static final int POLYAFTERTOUCH    = 0xa0;
        static final int CONTROLCHANGE     = 0xb0;
        static final int PROGRAMCHANGE     = 0xc0;
        static final int CHANNELAFTERTOUCH = 0xd0;
        static final int PITCHBEND         = 0xe0;
        static final int SYSTEMEXCLUSIVE   = 0xf0;

        // 7 bit controllers 
        static final int DAMPERPEDAL       = 0x40;
        static final int PORTAMENTO        = 0x41;
        static final int SOSTENUTO         = 0x42;
        static final int SOFTPEDAL         = 0x43;
        static final int GENERAL4          = 0x44;
        static final int HOLD2             = 0x45;
        static final int GENERAL5          = 0x50;
        static final int GENERAL6          = 0x51;
        static final int GENERAL7          = 0x52;
        static final int GENERAL8          = 0x53;
        static final int REVERB            = 0x5b;
        static final int TREMOLODEPTH      = 0x5c;
        static final int CHORUSDEPTH       = 0x5d;
        static final int DETUNE            = 0x5e;
        static final int PHASERDEPTH       = 0x5f;
        
        static final int METAEVENT = 0xff;
        //MetaEvent types
        static final int TEMPO             = 0x51;
        static final int ENDOFTRACK        = 0x2f;
        static final int SEQUENCENUMBER    = 0x00;
        static final int TEXTEVENT         = 0x01;
        static final int COPYRIGHTNOTICE   = 0x02;
        static final int SEQUENCENAME      = 0x03;
        static final int INSTRUMENTNAME    = 0x04;
        static final int LYRIC             = 0x05;
        static final int MARKER            = 0x06;
        static final int CUEPOINT          = 0x07;
        
        static final int CHANNELPREFIX     = 0x20;
        static final int SMPTEOFFSET       = 0x54;
        static final int TIMESIGNATURE     = 0x58;
        static final int KEYSIGNATURE      = 0x59;
        static final int SEQUENCERSPECIFIC = 0x74;
        
        static public void createShortEvent(int type, Track curtrack,int channelnum, int data1, int data2,long tickstime) {
           ShortMessage message = new ShortMessage();
           try {
               message.setMessage(type+channelnum, data1, data2); 
               MidiEvent event = new MidiEvent(message, tickstime);
               curtrack.add(event);
           } catch (Exception ex) { ex.printStackTrace(); }
        }
        static void createTempoEvent(long bpm,long tickstime,Track curtrack){
           MetaMessage message = new MetaMessage();
           long mtempo=60000000/bpm;
           byte[] data=new byte[3];
           data[0]=(byte)(0xff & (mtempo >> 16));
           data[1]=(byte)(0xff & (mtempo >> 8));
           data[2]=(byte)(0xff & (mtempo));
           try {
               message.setMessage(TEMPO, data, 3); 
               MidiEvent event = new MidiEvent(message, tickstime);
               curtrack.add(event);
               //tempoEvent=event;
           } catch (Exception ex) { ex.printStackTrace(); }
        }
        static void createProgChangeEvent(Track tr, int ch,int program,long tm){
        	createShortEvent(PROGRAMCHANGE,tr, ch, program,0,tm);
        }
        static void createNoteOnEvent(Track tr, int ch, int pitch, int vel, long tm){
                createShortEvent(NOTEON,tr, ch, pitch, vel,tm);
        }
	static void createNoteOffEvent(Track tr, int ch, int pitch, long tm){
                createShortEvent(NOTEOFF,tr, ch, pitch, 0,tm);
        }
        static void createNoteOnAndOff(Track tr, int ch, int pitch, int vel, long tm, long dur){
        	createShortEvent(NOTEON,tr, ch, pitch, vel,tm);
        	createShortEvent(NOTEOFF,tr, ch, pitch, 0,tm+dur);
        }
        static long tempoMessageToTempo(MetaMessage tempoMessage){
        
              byte[] data=tempoMessage.getData();
              long mtempo=0L;
              mtempo|=((data[0]&0xFF)<<16);
              mtempo|=((data[1]&0xFF)<<8);
              mtempo|=((data[2]&0xFF));
              long thistempo=(60000000/mtempo);
              return thistempo;
        
        }
        static int findMyNoteOff(Track tr,int noteOnIndex ){
        	MidiEvent onEvent=tr.get(noteOnIndex);
        	MidiMessage thisMessage=onEvent.getMessage();
			    
	        byte[] thisData=thisMessage.getMessage();
			    
                int thisStatus=thisData[0];
		int curPitch=thisData[1];
                int curChannel=thisStatus&0x0f;
        	int sz=tr.size();
        	int noteOffIndex=-1;
        	
        	int offidx=noteOnIndex+1;
		while(offidx<sz){
			MidiEvent testEvent=tr.get(offidx);
                			    
                	MidiMessage testMessage=testEvent.getMessage();
                        byte[] testData=testMessage.getMessage();
                	int testStatus=testData[0];
                	            
                        if((testStatus&0xf0)== NOTEOFF){
	                	if( (testStatus&0x0f)==curChannel){
	                	 // same channel
	                	        if(testData[1]==curPitch){
	                	        // same pitch
	                	           return offidx;
	                	        }
	                	 }
	                }
			offidx++;
		}
        	return noteOffIndex;
        
        }
        static void removeExtraProgramChanges(Sequence seq){
        	Track[] tracks=seq.getTracks();
        	int numtracks=tracks.length;
        	for(int t=0;t<numtracks;t++){
        		removeExtraProgramChanges(tracks[t]);
        	}
        }
        static void removeExtraProgramChanges(Track tr){
        	MidiEvent curEvent=null;
        	MidiMessage curMessage=null;
        	byte[] curData;
        	int lastProg=-1;
        	int curProg=-1;
        	int sz=tr.size();
        	int i=0;
        	while ( i<tr.size()){
        		curEvent=tr.get(i);
        		curMessage=curEvent.getMessage();
        		curData=curMessage.getMessage();
        		int curStatus=curData[0];
        		
        		if((curStatus&0xf0)==PROGRAMCHANGE){
        			curProg=curData[1];
        			
        			if(curProg==lastProg){
        			        
        				tr.remove(curEvent);
        				continue;
        			}
        			lastProg=curProg;
        			
        		}
        		i++;
        		
        	}
        }
        static Track insertProgramChanges(Track oldTrack, Track newTrack){
        	// returns the new track

        	int oldSz=oldTrack.size();
        	
        	MidiEvent curEvent=null;
        	MidiMessage curMessage=null;
        
        	byte[] curData;
        	
        	int curProg=0;

        	
           	ShortMessage[] curProgMessage={null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null};
           	

        	
        	for(int i=0;i<oldSz;i++){

        		curEvent=oldTrack.get(i);
        		curMessage=curEvent.getMessage();
        		curData=curMessage.getMessage();
        		int curStatus=curData[0];
        		int curChannel=curStatus&0x0f;
        		if((curStatus&0xf0)==PROGRAMCHANGE){
				        			
        			curProgMessage[curChannel]=(ShortMessage)curMessage;
        			        			
        		}
        		else if((curStatus&0xf0)==NOTEON){
        		        
        		        if(curProgMessage[curChannel]==null){
        		        	curProgMessage[curChannel] = new ShortMessage();
        		        	try {
               					curProgMessage[curChannel].setMessage(PROGRAMCHANGE+curChannel, curProg,0); 
               
                 			}    catch (Exception ex) { ex.printStackTrace(); continue;}
                 
                 		}
        			long curTick=curEvent.getTick();
				
               			MidiEvent pEvent = new MidiEvent(curProgMessage[curChannel], curTick);
               			newTrack.add(pEvent);
               			newTrack.add(curEvent);

        		}
        		else{
        			newTrack.add(curEvent);
        		}
        		
        		
        		
        	}
        	
        	return newTrack;
        }
        
        static MidiEvent getSequenceTempoEvent(Sequence seq){
        	MidiEvent curEvent=null;
        	MidiMessage curMessage=null;
        	
        	Track[] tracks=seq.getTracks();
        	Track tempoTrack=tracks[0];
        	int sz=tempoTrack.size();
        	for(int i=0;i<sz;i++){
        		curEvent=tempoTrack.get(i);
        		curMessage=curEvent.getMessage();
        		int status=curMessage.getStatus();
        		if(status==MetaMessage.META){
        		     MetaMessage message=(MetaMessage)curMessage;
        		     if (message.getType() == TEMPO) {
                		     
                                    
                                     return curEvent;
                                     
                              }
        		    
        		}
        	}
        	return null;
        }
        static int getSequenceTempo(Sequence seq){
        	MidiEvent tempoEvent=getSequenceTempoEvent(seq);
        	if(tempoEvent==null) return 0;
        	MetaMessage tempoMessage=(MetaMessage)tempoEvent.getMessage();
		byte[] data=tempoMessage.getData();
                long mtempo=0;
  	     	mtempo |= ((data[0]&0xFF)<<16);
  	     	mtempo |= ((data[1]&0xFF)<<8);
             	mtempo |= ((data[2]&0xFF));
                long thistempo=(60000000/mtempo);
                                     
                return (int) thistempo;
  		
        }
        static void setSequenceTempo(Sequence seq, int bpm){
        	MidiEvent tempoEvent=getSequenceTempoEvent(seq);
        	
        	Track [] tracks=seq.getTracks();
        	if(tracks.length>0){
        	
        			Track tempoTrack=tracks[0];
        			tempoTrack.remove(tempoEvent);
        			createTempoEvent((long) bpm,0,tempoTrack);
        	}
        	
        }
        static void setSequenceConductorLength(Sequence seq){
        // to make sure that track 0 length is as 'long' as the
        // rest of the tracks. KCS, for one, calls first track the
        // 'Conductor track' and assumes that the first track shows
        // how long the whole sequence is
        	Track [] tracks=seq.getTracks();
        	
        	long ticklength=seq.getTickLength();
        	if(tracks.length>0){
        		Track conductorTrack=tracks[0];	
        		int size=conductorTrack.size();
        		for(int i=0;i<size;i++){

        			MidiEvent mE=conductorTrack.get(i);        			
        			MidiMessage thisMessage=mE.getMessage();
        			if(thisMessage instanceof MetaMessage){

        				if(((MetaMessage)(thisMessage)).getType()==ENDOFTRACK){
        					
        					mE.setTick(ticklength);
        					
        				}
        			}
        			
        		}
        	
        	}
        }
        static long quantizeTick(long tick, int qTicks){
        // round tick to nearest qTicks
        	
        	long numQTick=tick/qTicks;
        	long prevQTick=numQTick*qTicks;
        	long rem=tick%qTicks;
        	if(rem>=qTicks/2){
        		return prevQTick+qTicks;
        	}
        	else{
        		return prevQTick;
        	}        	        	
        	
        }
        static Sequence quantizeSequence(Sequence oldSequence, Sequence newSequence, int qTicks,int qType){
        	// returns a quantized version of oldSequence
        	
        	Track[] tracks=oldSequence.getTracks();
        	int numTracks=tracks.length;
        	for(int tr=0;tr<numTracks;tr++){
        		Track track=tracks[tr];
        		int sz=track.size();
        		Track newTrack=newSequence.createTrack();
        		long lastOffTick=0;
        		
        		for(int i=0;i<sz;i++){
        			MidiEvent mE=track.get(i);
        			
        			MidiMessage thisMessage=mE.getMessage();
			    
			        byte[] thisData=thisMessage.getMessage();
			    
			        int thisStatus=thisData[0];
			    	int thisEventType=thisStatus & 0xf0;
                            	
        			
        			if(thisEventType==NOTEON){
        				int curPitch=thisData[1];
        		    		int curChannel=thisStatus&0x0f;
        		    		long oldOnTick=mE.getTick();
        		    		        				
        				// find the corresponding note off
        		    		int offidx=findMyNoteOff(track,i);
        		    		
        				MidiEvent theNoteOff=null;
        				if(offidx>0){
        					theNoteOff=track.get(offidx);
        				}
        				
					if(theNoteOff!=null){
					
        					MidiMessage onMessage=thisMessage;
                		    		long newOnTick=quantizeTick(oldOnTick, qTicks);
                				MidiEvent newEvent = new MidiEvent(onMessage, newOnTick);
                				newTrack.add(newEvent);	
                				
                				MidiMessage offMessage=theNoteOff.getMessage();
                		    		long oldOffTick=theNoteOff.getTick();
                		    		long oldDur=oldOffTick-oldOnTick;
                		    		long newOffTick=newOnTick+oldDur;                		    		
                		    		
                		    		switch(qType){
                		    		    case 0:
                		    		      	newOffTick=newOnTick+oldDur;
                		    		      	break;
                		    		    case 1:
                		    		      	newOffTick=newOnTick+quantizeTick(oldDur, qTicks);
                		    		      	if(newOffTick==newOnTick) 
                		    		      		newOffTick=newOnTick+qTicks;
                		    		      	break;
                		    		    case 2:
                		    		    	newOffTick=newOnTick+qTicks;
                		    		    	break;
                		    		    default:
                		    		    	newOffTick=newOnTick+oldDur;
                		    		   
                		    		}
                				if(newOffTick>lastOffTick)
                					lastOffTick=newOffTick;
                				newEvent = new MidiEvent(offMessage, newOffTick);
                				newTrack.add(newEvent);
                				
					}
        	
        			}
        			else if(thisEventType==NOTEOFF){
        			//deal with noteoffs with the noteon above
        			}
        			else if(thisEventType==PROGRAMCHANGE){
        				MidiMessage message=thisMessage;
        				long newTick=quantizeTick(mE.getTick(), qTicks);
        				MidiEvent newEvent = new MidiEvent(message, newTick);
               				newTrack.add(newEvent);
        				if(newTick>lastOffTick){
        					lastOffTick=newTick;
        				}
        			}
        			else if(thisMessage instanceof MetaMessage){
        				int type=((MetaMessage)(thisMessage)).getType();
        				MidiMessage message=thisMessage;
        				long newTick=0;
        				
        				if(type==ENDOFTRACK){
        					
        					newTick=lastOffTick;
        					
        				}
        				else{
        				        newTick=quantizeTick(mE.getTick(), qTicks);
        				}
        				MidiEvent newEvent = new MidiEvent(message, newTick);
        				newTrack.add(newEvent);
        			}
        			else{
        				MidiEvent newEvent=mE;
        				newTrack.add(newEvent);
        				
        			}
        			
        		}
        	}
        	return newSequence;
        }
        static void instrumentsToTracks(Sequence oldSequence, Sequence newSequence){
        
        	Track[] oldTracks=oldSequence.getTracks();
		int oldNumTracks=oldTracks.length;
		for(int i=0;i<oldNumTracks;i++){
		   Track oTr=oldTracks[i];
		   int oTrSz=oTr.size();
		   if(oTrSz<2)continue;
		   Track curTrack=newSequence.createTrack();
		   
		   Track[] iTracks=new Track[127];
		   int curProg=-1;
		   boolean initP=false;
		   for(int e=0;e<oTrSz;e++){
			MidiEvent oEvent=oTr.get(e);
			MidiMessage oMessage=oEvent.getMessage();
			    
			byte[] oData=oMessage.getMessage();
			    
			int oStatus=oData[0];
			int oEventType=oStatus & 0xf0;
			if(oEventType==PROGRAMCHANGE){
			    int thisProg=oData[1];
			    boolean newInstrument=false;
			    if(iTracks[thisProg]==null){
			    	if(initP){
			    		iTracks[thisProg]=newSequence.createTrack();
			    		
			    	}
			    	else{
			    		iTracks[thisProg]=curTrack;
			    		initP=true;
			    	}
			    	newInstrument=true;
			         
			    }
			    curTrack=iTracks[thisProg];
			    
			    
			    curTrack.add(oEvent);
			    curProg=thisProg;
			}
			else if(oEventType==NOTEON){
			    int noteOffIdx=findMyNoteOff(oTr,e);
			    
			    MidiEvent oNoteOffEvent=oTr.get(noteOffIdx);
			    curTrack.add(oEvent);
			    MidiMessage oOffMessage=oNoteOffEvent.getMessage();
			    long oOffTick=oNoteOffEvent.getTick();
			    // we have to construct a new event for the note offs because
			    // findMyNoteOff doesn't make sure that each noteon gets "its own"
			    // noteoff, it just finds the next one that fits the noteon.
			    
			    MidiEvent newNoteOffEvent  = new MidiEvent(oOffMessage, oOffTick);
			    curTrack.add(newNoteOffEvent);
			    
			
			}
			else if(oEventType==NOTEOFF){
			
			
			}
			else{
			    curTrack.add(oEvent);
			
			}
			
		   }
				
		}

        }
        static void removeUnusedTracks(Sequence seq){
		if(seq==null) return;
		Track[] tracks=seq.getTracks();
		int ntracks=tracks.length;
		for(int i=0;i<ntracks;i++){
			Track t=tracks[i];
			
			if(t.size()<2){
				seq.deleteTrack(t);
			}
				
		}
	}

}