import java.util.*;
import java.awt.*;


class StateVariable{
  
  int turtleVariableIndex;
  double mymin,mymax;
  
  private int variableType=0;
  private String token=null;
  private String name=null;
  
  
  public StateVariable(){
  
  }
  public StateVariable( int ti){
      turtleVariableIndex=ti;
  }
  public StateVariable( int ti, String tok, String nm){
      turtleVariableIndex=ti;token=tok;name=nm;
  }
  public StateVariable( int ti, String tok, String nm, int type){
      turtleVariableIndex=ti;token=tok;name=nm;
      variableType=type;
  }

  public StateVariable(StateVariable var){
      turtleVariableIndex=var.turtleVariableIndex;
      variableType=var.variableType;
      mymin=var.mymin;
      mymax=var.mymax;
      token=var.token;
      name=var.name;
  }
  public StateVariable( int ti,int t, double min, double max){
      turtleVariableIndex=ti;
      variableType=t;
      initMinMax(min,max);
  }
  void initMinMax(double min,double max){
  	mymin=min;mymax=max;
  }
  void initMinMax(double oneval){
  	mymin=oneval;mymax=oneval;
  }
  void MinMax(double compval){
  	if(compval<mymin)mymin=compval;
  	if(compval>mymax)mymax=compval;
  }
  double normedFromMin(double dd){
  	double vwide=mymax-mymin;
  	if(vwide!=0){
  	    return (dd-mymin)/vwide;
  	}
  	return 0;
  }
  double normedFromMidRange(double dd){
  	double midv=(mymin+mymax)/2;
  	double vwide=mymax-mymin;
        if(vwide!=0){
  	   return (dd-midv)/vwide;
  	}
  	return 0;
  }
  double normedFromMidRange(LState T){
  	double midv=(mymin+mymax)/2;
  	double vwide=mymax-mymin;
        if(vwide!=0){
  	   return (T.variables[turtleVariableIndex]-midv)/vwide;
  	}
  	return 0;
  }
  double getWidth(){
  	return (mymax-mymin);
  }
  double getMidRange(){
  	return (mymin+mymax)/2;
  }
  String getName(){
  	return name;
  }

  String getToken(){
  	return token;
  }
  public String toString(){
  	return getName();
  }

  String limitsToString(){
  	String limitsString  = token + " min="+mymin;
	limitsString += "   " + token + " max="+mymax;
	return limitsString;
  }
  boolean tokenEquals(String str){
  	return token.equals(str);
  }
}


class Map{
        //static final int MAXCONTROLLERVARIABLES=10;
        
        StateVariable pitch    =new StateVariable(Interpret.XVAR);
        StateVariable duration =new StateVariable(Interpret.DLENVAR);
        StateVariable volume   =new StateVariable(Interpret.THICKVAR);;
        double pspread=1.0;
        double dspread=1.0;
        double vspread=1.0;
        
        double dmultiplier=1.0;
        /* // unimplemented 
        StateVariable[] controls= new StateVariable[MAXCONTROLLERVARIABLES];
        byte[] controlnumber=new byte[MAXCONTROLLERVARIABLES];
        double[] controlspread=new double[MAXCONTROLLERVARIABLES];
        int numcontrols;
        */
        
        Scale scale=Scale.Major;
        int mode=0;// as in 'Locrian mode', etc        
        int transpose=0;
        
        
        
        public Map(){
        
        }
        public Map(StateVariable p,StateVariable d,StateVariable v){
        	pitch=p;
        	duration=d;
        	volume=v;
        }
        /*
        double getPitchValue(){
        
        
        }
	double getDurationValue(){
        
        
        }
        double getVelocityValue(){
        
        
        }
        */
}


public class LState{
	static long seed=System.currentTimeMillis();
        static Random prng=new Random(seed);

        static TYParse calculator=new TYParse();
        
        double deltaangle;
        // 'state variables'
        double[] variables=new double[Interpret.numStateVariables];
        double x,y,z;
        Vector3 fdirection;
        Vector3 udirection;
        Vector3 ldirection;
        double length=1.0;
        double drawlength=1.0;
        double thickness=10.0;
        
        int color=0;
        int transpose=0;
        double dfact=1.0;
        double vfact=1.0;

        long cumtime=0;
        long prevcumtime=0;
        int track=0;
        int inpoly=0;

        Map mymap;
        
        static final int XVAR=Interpret.XVAR,YVAR=Interpret.YVAR,ZVAR=Interpret.ZVAR,
	                 FXVAR=Interpret.FXVAR,FYVAR=Interpret.FYVAR,FZVAR=Interpret.FZVAR,
	                 UXVAR=Interpret.UXVAR,UYVAR=Interpret.UYVAR,UZVAR=Interpret.UZVAR,
	                 LXVAR=Interpret.LXVAR,LYVAR=Interpret.LYVAR,LZVAR=Interpret.LZVAR,
			 LENVAR=Interpret.LENVAR,DLENVAR=Interpret.DLENVAR, THICKVAR=Interpret.THICKVAR;
        public LState(){
           
           fdirection=new Vector3();
           udirection=new Vector3();
           ldirection=new Vector3();
           
        }
        public LState (LState orig){
        	deltaangle=orig.deltaangle;
        	x=orig.x; y=orig.y; z=orig.z;
        	for(int i=0;i<Interpret.numStateVariables;i++){
        	    variables[i]=orig.variables[i];
        	}
        	// copy the vectors
        	
        	fdirection=new Vector3(orig.fdirection);
        	udirection=new Vector3(orig.udirection);
                ldirection=new Vector3(orig.ldirection);
        	length=orig.length;
        	drawlength=orig.drawlength;
        	thickness=orig.thickness;
        	
        	color=orig.color;
        	transpose=orig.transpose;
        	
        	dfact=orig.dfact;
        	vfact=orig.vfact;

        	cumtime=orig.cumtime;
        	prevcumtime=orig.prevcumtime;
        	track=orig.track;
        	inpoly=orig.inpoly;

        	mymap=orig.mymap; // refer to the original map
        
        }
        double getStateVariableValue(StateVariable sV){
                double dret=0;
        	if(sV.turtleVariableIndex<0){
                       dret=calculator.getValu(sV.getName());                           
                }
                else{
                       dret=variables[sV.turtleVariableIndex];
                }
                return dret;
        }
        static void resetSeed(){
                prng=new Random(seed);
        	//prng.setSeed(seed);	
        }
        static void newSeed(){
        	seed=System.currentTimeMillis();
        	resetSeed();
        }
        void stateToVariables(){
        	positionToVariables();
        	vectorsToVariables();
        	lengthsToVariables();
        	//System.out.println(calculator.getList());
        }
        void lengthsToVariables(){
        	variables[LENVAR]=length;
        	variables[DLENVAR]=drawlength;
        	variables[THICKVAR]=thickness;
        	
        	calculator.setValu("length",length);
        	calculator.setValu("drawlength",drawlength);
        	calculator.setValu("thickness",thickness);
        	
        }
        void positionToVariables(){
        	variables[XVAR]=x;
        	variables[YVAR]=y;
        	variables[ZVAR]=z;
        	
        	calculator.setValu("x",x);
        	calculator.setValu("y",y);
        	calculator.setValu("z",z);
        	
        }
        void vectorsToVariables(){
                variables[FXVAR]=fdirection.x;
        	variables[FYVAR]=fdirection.y;
        	variables[FZVAR]=fdirection.z;
        	variables[UXVAR]=udirection.x;
        	variables[UYVAR]=udirection.y;
        	variables[UZVAR]=udirection.z;
        	variables[LXVAR]=ldirection.x;
        	variables[LYVAR]=ldirection.y;
        	variables[LZVAR]=ldirection.z;
	        
        	calculator.setValu("forwardx",fdirection.x);
        	calculator.setValu("forwardy",fdirection.y);
        	calculator.setValu("forwardz",fdirection.z);
        	calculator.setValu("upx",udirection.x);
        	calculator.setValu("upy",udirection.y);
        	calculator.setValu("upz",udirection.z);
        	calculator.setValu("leftx",ldirection.x);
        	calculator.setValu("lefty",ldirection.y);
        	calculator.setValu("leftz",ldirection.z);
        	
        }
        void variablesToVectors(){
        	fdirection.x=variables[FXVAR];
        	fdirection.y=variables[FYVAR];
        	fdirection.z=variables[FZVAR];
        	udirection.x=variables[UXVAR];
        	udirection.y=variables[UYVAR];
        	udirection.z=variables[UZVAR];
		ldirection.x=variables[LXVAR];
        	ldirection.y=variables[LYVAR];
        	ldirection.z=variables[LZVAR];

        }
        void normalizeDirections(){

        	fdirection.normalize();
        	udirection.normalize();
        	ldirection.normalize();

        }
        void orient(Vector3 f, Vector3 u, Vector3 l){
              fdirection=new Vector3(f);
              udirection=new Vector3(u);
              ldirection=new Vector3(l);

        }
        void orientAndPlace(double dA, Vector3 f, Vector3 u, Vector3 l, double x0, double y0, double z0){
                deltaangle=dA;
        	orient(f,u,l);
        	x=x0;y=y0;z=z0;
        }
  
        void rollToHorizontal(){

           if( fdirection.x != 0 && fdirection.z != 0){
                   ldirection=fdirection.cross(Vector3.Sky);
                   ldirection.normalize();                             
                   udirection=fdirection.cross(ldirection);
                   udirection.normalize();                                     
           }

        
        }  
        void turn(double a){

                fdirection=fdirection.rotate(udirection,a);
                ldirection=ldirection.rotate(udirection,a);
                normalizeDirections();

        }
        void pitch(double a){ // *not* musical pitch

                fdirection=fdirection.rotate(ldirection,a);
                udirection=udirection.rotate(ldirection,a);
                normalizeDirections();

        }
        void roll(double a){
                ldirection=ldirection.rotate(fdirection,a);
                udirection=udirection.rotate(fdirection,a);
                normalizeDirections();

        }
        void moveForward(double dist){
                x = x + dist * (fdirection.x);
                y = y + dist * (fdirection.y);
                z = z + dist * (fdirection.z);
        }
        void turnAround180(){//turn 180 degrees around up vector 
                fdirection.reverse();
                ldirection.reverse();
        }
        void rollAround180(){//turn 180 degrees around forward vector 
                udirection.reverse();
                ldirection.reverse();
        }
        void scaleLength(double sc){
        	length=length*sc;
        }
        void scaleAngle(double sc){
                deltaangle = deltaangle * sc;
        }
        void scaleThickness(double sc){
                thickness = thickness * sc;
        }
        void setColor(int c){
        	color=c;
        }
        void setTrack(int t){
        	track=t;
        }
        void setMap(Map m){
        	mymap=m;
        }
        void randomTurn(double maxangle){
                        
                double turn;
                
                turn = maxangle - 2 * prng.nextDouble() * maxangle;
                fdirection.xRot(turn);
		udirection.xRot(turn);
		ldirection.xRot(turn);

                turn = maxangle - 2 * prng.nextDouble() * maxangle;
                fdirection.yRot(turn);
		udirection.yRot(turn);
		ldirection.yRot(turn);

                turn = maxangle - 2 * prng.nextDouble() * maxangle;
                fdirection.zRot(turn);
		udirection.zRot(turn);
		ldirection.zRot(turn);
		normalizeDirections();
        }
        public String toString(){
            String retStr="";
            retStr+="x="+x+" y="+y+" z="+z+"\n";
            retStr+=" forward="+fdirection+" up="+udirection+" left="+ldirection+"\n"; 
            retStr+=" length="+length+" thickness="+thickness+" color="+color+"\n";
            retStr+=" transpose="+transpose+" dfact="+dfact+" vfact="+vfact+"\n";
            retStr+=" cumulativetime="+cumtime+" previous time="+prevcumtime+"\n";
            retStr+=" track="+track+" inpoly="+inpoly;
            return retStr;
        }
        

}

class LStack {
          private LinkedList list = new LinkedList();
          
          public void push(Object o) {
            list.addFirst(o);
          }
          
          public Object pop() { 
            try{
              return list.removeFirst();
            } 
            catch(NoSuchElementException e){
              return null;
            }
          }
          
          public Object top() { 
             try{
                return list.getFirst(); 
             }
             catch(NoSuchElementException e){
                return null;
             }
          }         

}

class TurtleStack extends LStack{

	void pushTurtle(LState turtle){

	    LState s=new LState(turtle);
	    push(s); // what actually goes on the stack is a copy	    
	}
	
	LState popTurtle(){
	    return (LState)pop();
	}
} 

