import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import javax.sound.midi.*;

class MIDICanvas extends JComponent
{
        private Vector lines;
        private Color[] colors=DrawCanvas.colors;

        private int numColors=colors.length;
        
        //Sequence sequence=null;
        
        long seqLength;
        
        public MIDICanvas(){
        	//super();
        	//setBackground(new Color(157,157,209));
        	//setBackground(new Color(255,255,255));
        	setBackground(new Color(0,0,0));
        	setOpaque(true);
        	lines=new Vector();
        }
        void clearLines(){
        	lines.removeAllElements();
        }
        void setSequence(Sequence seq){
             //sequence=seq;
             clearLines();
             paintSequence(seq);
             update(getGraphics());
        }
        public void paint(Graphics g){
 
  	    int w=getWidth(),
                h=getHeight(); 

            g.setColor(getBackground());
            g.fillRect(0,0,w,h);
  	    drawNotes(g);
             
        }
        void drawNotes(Graphics g){
        
            int sz=lines.size();
  	    
       	    for(int i=0;i<sz;i++){
       	    	((NoteLineObject)lines.get(i)).draw(g);
       	    }
       	     	
        }
	void paintTrack(Track track){
	
		int nEvents=track.size();
			
		int curProg=-1;
		for(int e=0;e<nEvents;e++){
			long tickLength=0;
			long thisTick=0;
			    
			MidiEvent thisEvent=track.get(e);
			    
			MidiMessage thisMessage=thisEvent.getMessage();
			    
			byte[] thisData=thisMessage.getMessage();
			    
			int thisStatus=thisData[0];
			    
    			if((thisStatus&0xf0)==MIDIEventStuff.PROGRAMCHANGE){
				curProg=thisData[1];
		    	}
    			else if( (thisStatus&0xf0)==MIDIEventStuff.NOTEON){
		    		int curPitch=thisData[1];
		    		int curChannel=thisStatus&0x0f;
		    		thisTick=thisEvent.getTick();	
		    			    		
		    		MidiEvent offEvent=null;
		        	int offidx=MIDIEventStuff.findMyNoteOff(track,e);
		        	if(offidx>0){
		        		offEvent=track.get(offidx);
		        
		        		long offTick=offEvent.getTick();
		        		tickLength=offTick-thisTick;
		        		NoteLineObject lE=new NoteLineObject(thisTick,thisTick+tickLength,curPitch,curProg);
					lines.addElement(lE);
		        	}
		        

		    	}// end of this is a note on
		    
			    
		}// end of for each event in track	
	
	}        
	void paintSequence(Sequence seq){
		// get a list of notes
		seqLength=seq.getTickLength();
		Track[] tracks=seq.getTracks();
		
		
		int tr=0;
		for (tr=0;tr<tracks.length;tr++){
			paintTrack(tracks[tr]);
		}
	
	}



        public Dimension getPreferredSize() {
            return new Dimension(200,128);
        }
        public Dimension getMinimumSize() {
            return new Dimension(200,128);
        }
        public Dimension getMaximumSize() {
            return getPreferredSize();
        }
        class NoteLineObject{
        	long x1,x2;
        	int pitch;
        	int color;
        	
        	public NoteLineObject(long xs,long xe,int p,int c){	
        		x1=xs;
        		x2=xe;
        		pitch=p;
        		color=c;
        	
        	}
        	void draw(Graphics g){
        		
                        
                        if(g==null) return;
                        
                        int w=getWidth();
                 	int h=getHeight();
                 	
                        double wscale=((double)(seqLength))/((double)w);
                 	double hscale=((double)(128))/((double)h);
                 	int ix1,ix2,iy1;
                        
                 	double x1s=x1;
                 	double x2s=x2;
                 	double y1s=pitch;                 	
                 	                 	
                        ix1=(int) (x1s/wscale);
                        iy1=(int) (h/2-(pitch-64)/hscale);
                        ix2=(int) (x2s/wscale);
                       
                        g.setColor(colors[color%numColors]);
                  	g.drawLine(ix1,iy1,ix2,iy1);
        		
        	}
        }
}

