
import java.util.*;
import java.io.IOException;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

abstract class NumberInputField extends JPanel implements KeyListener{
      JTextField myTF;
      JLabel myLabel;
      char [] badChars={'`','`','!','@','#','$','%','^','&','*','(',')','_','=','|','\\','{','}',
                        '[',']',':',';','"','\'','<',',','>','?','/'};
      int badCharsLen=badChars.length;
      boolean minmaxset=false;
      public NumberInputField(String L){
             myLabel=new JLabel(L);
             myTF=new JTextField("");
             add(myLabel);
             
             add(myTF);
             myTF.addKeyListener(this);
      }
      public NumberInputField(String L,int cols){             
             myLabel=new JLabel(L);
             myTF=new JTextField("",cols);
             add(myLabel);
             
             add(myTF);
             myTF.addKeyListener(this);
      }
      public NumberInputField(String L,boolean bordered){
             myLabel=new JLabel(L);
             myTF=new JTextField("");
             add(myLabel);
             if(bordered)
                setBorder(BorderFactory.createLineBorder(new Color(159,159,159)));
                
             add(myTF);
             myTF.addKeyListener(this);
      }
      public NumberInputField(String L,int cols, boolean bordered){
             myLabel=new JLabel(L);
             myTF=new JTextField("",cols);
             add(myLabel);
             if(bordered)
                setBorder(BorderFactory.createLineBorder(Color.gray));
                
             add(myTF);
             myTF.addKeyListener(this);
      }
      public void keyTyped(KeyEvent keyEvent){
          char c=keyEvent.getKeyChar();
          boolean bad=false;
          for(int i=0;i<badCharsLen;i++){
          	if(c==badChars[i]){
          	   bad=true;
          	   break;
          	}
          }
          if (bad || Character.isLetter(c)|| !validateChar(c)) {
            java.awt.Toolkit.getDefaultToolkit().beep();
            keyEvent.consume();
          }    	
             
      }
      public void keyPressed(KeyEvent keyEvent){}
      public void keyReleased(KeyEvent keyEvent){}
      
      public void addActionListener(ActionListener l){
            myTF.addActionListener(l); 
      } 
      abstract boolean validateChar(char c);


}
class IntInputField extends NumberInputField{

      private int myvalue;
      private int mymin,mymax;

      	
      public IntInputField(String L,int val){
             super(L);             
             myTF.setText(String.valueOf(val));
             myvalue=val;                          
      }
      public IntInputField(String L,int val,boolean bordered){
             super(L,bordered);             
             myTF.setText(String.valueOf(val));
             myvalue=val;                          
      }

      public IntInputField(String L,int val, int cols){
             super(L,cols);
             myTF.setText(String.valueOf(val));
             myvalue=val;                          
      }
      public IntInputField(String L,int val, int cols,boolean bordered){
             super(L,cols,bordered);
             myTF.setText(String.valueOf(val));
             myvalue=val;                          
      }
      public int getValue(){
           try{
               myvalue=Integer.valueOf(myTF.getText()).intValue();
           }
           catch(Exception e){
                System.out.println("bad integer");
                try{
                    myvalue=(int)Double.valueOf(myTF.getText()).doubleValue();
                }
                catch(Exception e2){
                   System.out.println("bad number");
                }
           }
           
           if(minmaxset){
             if(myvalue>mymax){
                setValue(mymax);
                myvalue=mymax;
             }
	     if(myvalue<mymin){
                setValue(mymin);
                myvalue=mymin;
             }
           }
           return myvalue; 
      }
      public void setValue(int val){
           myvalue=val;
           myTF.setText(String.valueOf(val));
      }
      void setMinMax(int min, int max){
            mymin=min;mymax=max;
            minmaxset=true;
      }
      boolean validateChar(char c){

      	return true;
      
      }
}

class FloatInputField extends NumberInputField{

      private float myvalue;
      private float mymin,mymax;

      public FloatInputField(String L,float val){
             super(L);
             myTF.setText(String.valueOf(val));
             myvalue=val;             
      }
      public FloatInputField(String L,float val,int cols){
             super(L,cols);
             myTF.setText(String.valueOf(val));
             myvalue=val;
      }
      public float getValue(){
           myvalue=Float.valueOf(myTF.getText()).floatValue();
           if(minmaxset){ 
             if(myvalue>mymax){
                setValue(mymax);
                myvalue=mymax;
             }
	     if(myvalue<mymin){
                setValue(mymin);
                myvalue=mymin;
             }
           }
           return myvalue; 
      }
      public void setValue(float val){
           myvalue=val;
           myTF.setText(String.valueOf(val));
      }
      void setMinMax(float min, float max){
            mymin=min;mymax=max;
            minmaxset=true;
      }
      boolean validateChar(char c){

      	return true;
      
      }

}

class DoubleInputField extends NumberInputField{
      private double myvalue;
      private double mymin,mymax;

      public DoubleInputField(String L,double val){
             super(L);
             myTF.setText(String.valueOf(val));
             myvalue=val;             
      }
      public DoubleInputField(String L,double val,int cols){
             super(L,cols);
             myTF.setText(String.valueOf(val));
             myvalue=val;                          
      }
      public double getValue(){
           myvalue=Double.valueOf(myTF.getText()).doubleValue();
           if(minmaxset){ 
             if(myvalue>mymax){
                setValue(mymax);
                myvalue=mymax;
             }
	     if(myvalue<mymin){
                setValue(mymin);
                myvalue=mymin;
             }
           }
           return myvalue; 
      }
      public void setValue(double val){
           myvalue=val;
           myTF.setText(String.valueOf(val));
      }
      void setMinMax(double min, double max){
            mymin=min;mymax=max;
            minmaxset=true;
      }
      boolean validateChar(char c){

      	return true;
      
      }	
}
