
import java.util.*;
import java.io.*;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

//-----------------------------------------------------
class Production {
        private StringBuffer contextToIgnore;
        private int ignlength;
    	
    	private int recursenumtimes;
        private String in,out,fini;
        
        int numdone=0;
        boolean interruptme=false; // flag set by calling thread
        //--- Constructor -------------------------------------
        public Production(String A,int r,String cTI){
             if(A==null) in="";
             else
                in=new String(A); //make a copy of A
             out="";
             recursenumtimes = r;
             
             interruptme=false;
             setContextToIgnore(cTI);
        }
        // Tell the production process to quit
	public void interrupt(){
	     interruptme=true;
	}
	// Note that Production class does not a have its own thread
	// but this is still called 'run'
        public int run(){
                String hold;                
		StringBuffer NBuf=new StringBuffer("");
		numdone=0; 
                for(int n=0;n<recursenumtimes;n++){
                    int ilength=in.length();
                    String inC,Rt,Lft;
                    String replacementStr;
                    int c=0;
                    
                    while (c<ilength){
                    
                        Thread.yield(); // mainly to see if we are be asked
                                        // to stop
                        if(interruptme){// Were we asked to stop?
                           fini=in;
                           return numdone; // just leave from here

                        }
                        c=LTools.getNumericChars(in,c,NBuf);
                        if(NBuf.length()>0){
                           // if we got a numerical expression, just
                           // tack it onto the output...
                           out=out.concat(NBuf.toString());

                        }
                        else if(c<ilength){ // ... otherwise parse this character as
                                            // an L-system symbol
                          inC=in.substring(c,c+1);
                          Lft=removeContextToIgnore(in.substring(0,c)); 
                          Rt=removeContextToIgnore(in.substring(c+1,ilength));
                            
                          replacementStr=Rule.rules.getReplacementString(inC,Lft,Rt);
                        
                          out=out.concat(replacementStr);
                          c++;
                        }
                        
                    }
                    
                    fini=out;
                    
                    in=out; out="";
                    numdone++;
                }
                hold=in; in=out; out=hold;
                return numdone;
                //writeProduction("last.prd");               
                
        }
    
        int currentlength(){
                // return the length of production string so far
                // (that is, completed recursion length)
        	return fini.length();
        }
        void setContextToIgnore(String toIgnore){
        	contextToIgnore=new StringBuffer(toIgnore);
        	ignlength=contextToIgnore.length();
        }
        String getProduction(){
                 return fini;       
        }

        String removeContextToIgnore(String str){
        // a 'utility' to remove the characters that should be ignored
        // in determining context from str

                str=LTools.removeNumericalContext(str);// remove numerical expressions
                StringBuffer Buf=new StringBuffer(str);
                
                
                for(int i=0;i<ignlength;i++){
                       char c=contextToIgnore.charAt(i);
                       int j=0;
                       
                       while (j<Buf.length()){
                           if(c==Buf.charAt(j))
                               Buf.deleteCharAt(j);
                           else j++;
                       }                  
                }
                return(Buf.toString());
        }
                        
        
        
        boolean writeProduction(String fname){
             File f=new File(fname);
             FileWriter fw;
             try{
                fw=new FileWriter(f); 
             }catch(Exception e){return false;}
             try{
                 fw.write(fini);
             }catch(Exception e){return false;}
             try{
                 fw.close();
             }catch(Exception e){return false;}
             return true;
        }
}
//-----------------------------------------------------
// The RuleTables (KeyRuleTable, ContextRuleTable, and StochasticRuleTable)
// are only used in making the production string. They are remade for each
// new run. There are no methods for changing them or their constituents. 
// Instead, we 'cut them  loose' by dereferencing, and rebuild from scratch. 
class KeyRuleTable extends HashMap{
      
      synchronized void add(Rule rl){
           put(rl.key,rl.replacement);
      }
      synchronized String getReplacementString(String k,String Lft, String Rgt){
           String returnstring=k;
           Replacement replacement=(Replacement)Rule.rules.get(k);
           
           if(replacement!=null){
                      returnstring=replacement.getReplacementString(Lft,Rgt);              
           }
           if(returnstring==null) returnstring=k;
           return returnstring;
   }
}

//-----------------------------------------------------
class ContextRuleTable extends HashMap{

      void add(String context,Replacement replacement){                                
           	put(context,replacement);
           	// hashtable key is the context string
      }
      
      String getReplacementString(String Lcontext,String Rcontext){
           // we are looking for the hashtable key that matches
           // the given left and right contexts
           int Llen=Lcontext.length();
           int Rlen=Rcontext.length();
           String returnString=null;
           
           String contextString;  // the string we will construct
                                  // from Lcontext, Rcontext to match
                                  // the hashtable key
           
           // The context-keys are arbitrary length. Thats why the 
           // Lcontext and Rcontext strings passed to getReplacementString
           // are the entire left and right sides of the production string.
           // Want to properly match to such context-keys as 
           //	"B "      ( rulekey: B<A) B to the left, anything on the right
           //   " B"      ( rulekey: A>B) B to the right, anything on the left
           //   "B C"     ( rulekey: B<A>C) B on left, C on right
           //   "] B"     ( rulekey: <A>B)  B on right, empty on the left
           //   "]X ["    ( rulekey: ]X<A>) X on the left, empty on the right
           //   "] ["     ( rulekey: <A>)   empty on both sides 
           //   "BC DEF"  (rulekey: BC<A>DEF)  arbitrary context lengths
           
           // Since the contexts can be arbitrary lengths, we have to 
           // examine the context-keys (rather than just pass the given
           // given context to the hash table)... 
           Set s=keySet();// ... make an iterator on the context-keys
           Iterator it = s.iterator();
           while (it.hasNext()){
                   String contextKey=(String)it.next();// get this context-key
                   String cKLeft="",cKRight=""; 
                   // we are constructing left and right
                   // right context strings from this context key
                                       
                   int contextKeylen=contextKey.length();
                   int i=0;
                   while(contextKey.charAt(i)!=' '&&i<contextKeylen){
                        // left context is everything up to ' ' (space)
                        cKLeft+=contextKey.charAt(i);
                        i++;
                   }
                   i++;// skip over the space that separates left from right
                   while(i<contextKeylen){
                        // right context is everything in this key from the
                        // delineating ' '(space) to the end
                        cKRight+=contextKey.charAt(i);
                        i++;
                   }                   
                   
                   int cKLeftLen=cKLeft.length();
                   int cKRightLen=cKRight.length();
                   // cKLEnd and cKREnd tell whether this context-key 
                   // is meant to match to the end(s) of a
                   // production string
                   boolean cKLEnd = cKLeftLen>0  ? (cKLeft.charAt(0)==']'):false;
                   boolean cKREnd = cKRightLen>0 ? (cKRight.charAt(cKRightLen-1)=='['):false;
                   

                   if(Llen>=cKLeftLen && cKLeftLen>0){
                       
                       // if this context-key is supposed to match 'nothing' 
                       // on the left but there *is* something to the left,
                       // skip this key
                       if(cKLEnd){
                         continue;
                       }
                   }
                   if(Rlen>=cKRightLen && cKRightLen>0){
                       // if this context-key is supposed to match 'nothing' 
                       // on the right but there *is* something on the right,
                       // skip this key
                      if(cKREnd){
                         continue;
                      }
                   }
                   // now we will use the given context to make a 
                   // string to match to the context-key
                   contextString="";
                 
                   if(Llen>=cKLeftLen){
                       // if there's enough on the left to match
                       contextString=Lcontext.substring(Llen-cKLeftLen,Llen);
                   
                   }
                   else if (Llen==cKLeftLen-1 && cKLEnd){
                      // if the given left context is too short,
                      // that means the production begins with the current 
                      // context. cKLEnd is true when the context-key
                      // is to match the beginning of the production, and
                      // cKLEnd is signalled in the context-key by a ']'
                      // character appended to the front, so it should
                      // be one character longer than the given left context.
                      contextString="]"+Lcontext.substring(Llen-(cKLeftLen-1),Llen);
                   }
                   contextString=contextString+" ";
                   if(Rlen>=cKRightLen){
                       contextString=contextString+Rcontext.substring(0,cKRightLen);
                   }
                   else if (Rlen==cKRightLen-1 && cKREnd){
                      // The right side is short so we append '[' to match 
                      // end of production string context-key(s)
                      contextString=contextString+Rcontext.substring(0,cKRightLen-1)+"[";
                   }  
                   
                   Replacement r=null;
                   // Now, does this key actually match the given context?
                   if(contextString.equals(contextKey))
                        r=(Replacement)get(contextKey);
                   if(r!=null){           
                   	return r.getReplacementString(Lcontext,Rcontext);
                   }
                   // if not, loop through the next context-key
                   
           }
           // Didn't find a match
           return returnString;
      }
      
}
//-----------------------------------------------------
class StochasticRuleTable extends HashMap{
      private Random prng;      
      private float lasttotal;
      
      public StochasticRuleTable(){
            prng=new Random(System.currentTimeMillis());
            lasttotal=0;
      }
      void add(float p,Replacement r){// the key is a Float
           put(new Float(p+lasttotal),r);
           // the hash key is the probability accumulated so far
           lasttotal+=p;
      }
      
      String getReplacementString(String Lft,String Rgt){
           
           float r=prng.nextFloat();
           if(r>lasttotal){// no 'match'
                return (String)null;
           }
           float sum=0;
           float norm=1;
           Float k=null;
           Set s=keySet();
           Iterator it = s.iterator();
           
           float minsum=lasttotal;
           Float Fminsum=null;
           // we go through the entire table, looking for the entry
           // whose accumulated probability is the minimum one greater than r           
           while(it.hasNext()){
               k=(Float)it.next();
               sum=(k.floatValue())/norm;

               if(sum>r){
                   if(sum<=minsum){
                       minsum=sum;
                       Fminsum=k;
                   }
                    
               }
               
           }
           if(k!=null && Fminsum!=null){
              return ((Replacement)get(Fminsum)).getReplacementString(Lft,Rgt);
              
           }
           return (String) null;
             
      }

}

//-----------------------------------------------------
class Replacement{
   // each 'Replacement' can 'point' to a ContextRuleTable (a hashtable of
   // contexts/Replacements to match), or a StochasticRuleTable (a hashtable
   // of accumulated probabilities/Replacements), or a replacement string.
   // Each of the RuleTables is a HashMap of Replacements, so,
   // like a linked list, a chain of Replacements could go on and on
   // e.g. Key->Replacement->ContextRuleTable->Replacement->StochasticRuleTable->Replacement
   //      ->ContextRuleTable-> etc.   But the Rule/Replacement constructors limit the
   // actual possibilities.
   // By following the 'links' from Replacement to Replacement, we should find
   // a dead end. ie, a Replacement with no ContextRuleTable and no StochasticTable.
   // The replacement string found there is what we want to give back as the
   // the replacement. 
   
   ContextRuleTable myContextTable  =null;
   StochasticRuleTable myStochasticTable=null;
      
   private String replacementString;     

   public Replacement(String r){ // the default replacement string for this Replacement
       replacementString=new String(r);
   }
   public Replacement(String r,float p){
   
      replacementString = null;
      
      myStochasticTable=new StochasticRuleTable();      
      myStochasticTable.add(p,new Replacement(r));   
   }
   
   public Replacement(String context, String r){
      replacementString=null;

      myContextTable=new ContextRuleTable();
      myContextTable.add(context,new Replacement(r));
   }
   public Replacement(String context, String r,float p){
      replacementString=null;

      myContextTable=new ContextRuleTable();
      myContextTable.add(context,new Replacement(r,p));
   } 
   
   public Replacement(String r,float p,String context){
      replacementString = null;
            
      myStochasticTable=new StochasticRuleTable();      
      myStochasticTable.add(p,new Replacement(context,r));   
   }  
   
   void setReplacementString(String str){
       replacementString=str;
   }
   
   String getReplacementString(String LeftContext,String RightContext){
       String returnstring=null;
       // test in order of 'priority'
       if(myStochasticTable!=null){
      		returnstring=myStochasticTable.getReplacementString(LeftContext,RightContext);
       }
       if(returnstring==null&&myContextTable!=null){
      		returnstring=myContextTable.getReplacementString(LeftContext,RightContext);              		
       }
       if(returnstring==null){// no matching context, no stochastic table (or no string
                              // returned), so get the default replacement string if
                              // there is one
          	returnstring=replacementString;
       }

       return returnstring;    
   }
}
//-----------------------------------------------------
class Rule{
   static KeyRuleTable rules=new KeyRuleTable();
      
   String key;
   Replacement replacement=null;
   String ruleString=null;
   
   static void reinit(){
        // Cut the old KeyRuleTable loose to start over
        rules=new KeyRuleTable();
   }
   public Rule(String k, String r){
      // the basic rule: eg F=AB, replaces k with r
      key         = new String(k);
      if(!rules.containsKey(key)){
            replacement = new Replacement(r);
            rules.add(this);    
      }
      else{
         Replacement myreplacement=(Replacement)rules.get(key);
         myreplacement.setReplacementString(r);
      }
      ruleString= key+"="+r;
   }
   public Rule(String k, String r,float p){
      // stochastic rule, (.3)F=AB, replaces k with r, p of the time
      key         = new String(k);
      if(!rules.containsKey(key)){
            replacement = new Replacement(r,p);
            rules.add(this);    
      }
      else{
            Replacement myreplacement=(Replacement)rules.get(key);
            if(myreplacement.myStochasticTable==null){
                  myreplacement.myStochasticTable=new StochasticRuleTable();
            }
            myreplacement.myStochasticTable.add(p,new Replacement(r));
            
      } 
      ruleString= "("+p+")"+key+"="+r;
   }

   public Rule(String k,String r, String context){
      // context sensitive rule, A<F>B=CDE, context is given in the form
      // "A B" (left context is separated from right context by a space),
      //  k is replaced by r if k's context is 'context'
      key         = new String(k);
      if(!rules.containsKey(key)){
            replacement = new Replacement(context,r);
            rules.add(this);
      }
      else{
         
            Replacement myreplacement=(Replacement)rules.get(key);
          
            if(myreplacement.myContextTable==null){
            
                  myreplacement.myContextTable=new ContextRuleTable();
            }
            myreplacement.myContextTable.add(context,new Replacement(r));
            
      }       
      ruleString=contextToString(context)+"="+r;
   }
   public Rule(String k, String r,String context,float p){
      // A<F>B(.3)=CD, replace 'k' with 'r' if context is 'context',
      // p of the time
      
      key         = new String(k);
      if(!rules.containsKey(key)){
            replacement = new Replacement(context,r,p);

            rules.add(this);
      }
      else{
         
            Replacement myreplacement=(Replacement)rules.get(key);
          
            if(myreplacement.myContextTable==null){
            
                  myreplacement.myContextTable=new ContextRuleTable();
            }
            myreplacement.myContextTable.add(context,new Replacement(r,p));
            
      }
      ruleString=contextToString(context)+"("+p+")"+"="+r;

   }
   public Rule(String k, String r,float p, String context){
      // (.3)A<F>B=CD 
      // matches with F 30% of the time, *then* checks for context match

      key         = new String(k);
      if(!rules.containsKey(key)){
            replacement = new Replacement(r,p,context);
            rules.add(this);    
      }
      else{
            Replacement myreplacement=(Replacement)rules.get(key);
            if(myreplacement.myStochasticTable==null){
                  myreplacement.myStochasticTable=new StochasticRuleTable();
            }
            myreplacement.myStochasticTable.add(p,new Replacement(context,r));
            
      } 

      ruleString="("+p+")"+contextToString(context)+"="+r;
   }
   String contextToString(String context){
   
      String leftcontext=null,rightcontext=null;
    
      int L=context.length();
      int i;
      for(i=0;i<L;i++){
          if(context.charAt(i)==' '){
            break;
          }
      }
      if(i==0){leftcontext="";}
      if(L>0&&i==L-1){rightcontext="";}
      if(leftcontext==null){
         leftcontext=context.substring(0,i);
      }
      if(rightcontext==null){
         rightcontext=context.substring(i+1,L);
      }
      
      if(leftcontext.length()>0){
          if(leftcontext.length()==1 && leftcontext.charAt(0)==']')
             leftcontext="";
          leftcontext=leftcontext+"<";
          
      }
      if(rightcontext.length()>0){
           if(rightcontext.charAt(0)=='[')
             rightcontext="";

           rightcontext=">"+rightcontext;
      }
      return leftcontext+key+rightcontext;
   
   }
   
   static Rule stringToRule(String s){
      
      String LContext=null,RContext=null,K=null;
      String LeftSide=null,RightSide=null;
      // initialize character indexes to -1
      int p1Rindex=-1,p2Lindex=-1,LCindex=-1,RCindex=-1,LKindex=-1,RKindex=-1;
      // probabilities initialized to -1
      float p1=-1,p2=-1; 
      int L; //length of 'left side' 
      // (the right side is taken literally so we don't care about the length of the right side)
      
      // spaces have no meaning in a rule def
      s=LTools.removeSpaces(s);
      // We will separate the rule into LeftSide ('key') and RightSide ('replacement')
      // at '='
      StringTokenizer st=new StringTokenizer(s,"=");
      
      if(st.hasMoreTokens()){
          LeftSide=st.nextToken();
      }
      else{ // 'syntax' error
      }
      if(st.hasMoreTokens()){
          //RightSide=st.nextToken();
          RightSide="";
          while(st.hasMoreTokens()){
          	RightSide+=st.nextToken();
          	if(st.hasMoreTokens()){
          	   RightSide+="=";
          	}
          }
      }
      else{ // if RightSide is empty, the replacement is empty.
          RightSide="";
      }
                 
      L=LeftSide.length();
      // A buffer for gathering numerical values (probabilities)
      StringBuffer NBuf=new StringBuffer("");

      
      // march through the LeftSide string and find probabilites
      int i=0; // 'i' will be our current position in the string
      int p=0; // 'p' will keep track of where getNumericChars left off in the string
      while(i<L){
         // check each place to see if a number is there
         p=LTools.getNumericChars(LeftSide,i,NBuf);
         // getNumericChars returns (p) the location in the string where it found the first
         // character it doesn't consider to be part of a number.
         

         //if(LeftSide.charAt(p)==')') p++;
         // if NBuf has anything in it, get a probability for this rule
         if(NBuf.length()>0){
            if(i==0){// the string started with the probability
                     //p1=Float.parseFloat(NBuf.toString());
                     p1=(float)LTools.expressionParser.getValu(NBuf.toString());
                     // 'p1Rindex' tells last character of this probability
                     // This is used to delineate the LeftContext and/or the Key
                     p1Rindex=p;
            }
            else{ // the probability was not at the start
                     //p2=Float.parseFloat(NBuf.toString());
                     p2=(float)LTools.expressionParser.getValu(NBuf.toString());
                     // 'p2Lindex' tells where the first character of this probability is
                     // This is used to delineate the RightContext and/or the Key
                     p2Lindex=i;
            }
            i=p;
         }
         else i++;  // if no number found at 'i', go to next character
         
      }
      // find the context strings 
      //      
      String LNoNum=LTools.removeNumericalContext(LeftSide);
      int LNoNumLen=LNoNum.length();
      if(LNoNumLen>1){    
        LCindex=LNoNum.indexOf('<',0);
        RCindex=LNoNum.indexOf('>',0);
      }
      else{
      	LCindex=-1;RCindex=-1;
      }
      
      if(LCindex>-1){ // if '<' was found          
          LContext=LNoNum.substring(0,LCindex);
          // if the '<' was 'pointing' to nothing it is meant to match
          // "" context (ie, the beginning of production, rather than 'no context' or wild)
          if(LContext.length()==0) 
             LContext="]";
          // LKindex is first character position of the 'Key'   
          LKindex=LCindex+1;
      }
      else{ 
          
          LKindex=0;
      }
      
      
      
      if(RCindex>0){
          RContext=LNoNum.substring(RCindex+1, LNoNumLen);
          if(RContext.length()==0){ // the '>' was pointing at nothing, ie, it is meant
                                   // to match the end of a production
             RContext="[";
          }   
          // the key ends where the right context begins
          RKindex=RCindex;
      }
      else RKindex=LNoNumLen; // otherwise, no right context and no right end probability,
                      // the key ends at the end of the string
      if(LKindex<0||RKindex<0) return null;
      K=LNoNum.substring(LKindex,RKindex);
      if(K!=null){
          // build a context string to pass to the Rule constructor 
          String contextstring=new String("");
          if(LContext!=null){
             contextstring=contextstring+LContext;
          } 
          contextstring = contextstring+" "; // a space separates the left context
          				     // from the right context	
          if(RContext!=null){
             contextstring=contextstring+RContext;
          }
          // choose the correct constructor and return
          if(contextstring.length()>0&& !contextstring.equals(" ")){

             if(p1>0){ 
               return new Rule(K,RightSide,p1,contextstring);
             }
             else if(p2>0){
               return new Rule(K,RightSide,contextstring,p2);
             }
             else
               return new Rule(K,RightSide,contextstring);
          }
          else if(p1>0 ||p2>0){
              return new Rule(K,RightSide,p1>0?p1:p2);
          }
          
          
          return new Rule(K,RightSide);
      }
      return null;
      
   }
   

      
   public String toString(){
      
      		return ruleString;
   }
   static String getLeft(Rule r){

		String rulestring=r.ruleString;
		int L=rulestring.length();
		int i=0;
		while(i<L){
		   if(rulestring.charAt(i)=='='){
		      return rulestring.substring(0,i);
		   } 
		   i++;
		}
		return "";
   }
   static String getRight(Rule r){
		String rulestring=r.ruleString;
		int L=rulestring.length();
		int i=0;
		while(i<L){
		   if(rulestring.charAt(i)=='='){
		      if(i<L-1)
		         return rulestring.substring(i+1,L);
		      else return "";
		   } 
		   i++;
		}
		return "";
   }
   static boolean checkRuleString(String str){
	   // check for ok syntax. There's not really much to check
	   // for. Just about any string containing an '=' to the
	   // right of a non-whitespace character passes
	   boolean ret=false;
	   str=LTools.removeSpaces(str);
	   int equalsidx=str.indexOf('=');
	   int commentidx=str.indexOf('#');
	   if(equalsidx<1) return false; // no '=', or nothing in front of it
	   else if(commentidx>=0 && equalsidx>commentidx){
	       // there's an '=', but its hidden by a comment       
	             return false;
	   }
	   else ret=true;
	   
	   return ret;
   }
}



class RulesPanel extends JPanel implements ActionListener{

        private JTextArea rulesText;
	private JScrollPane rulesScrollPane;
	private JButton mutateb;
	private JButton undoMutateb;
	private Mutator mutator=new Mutator();
	
	RulesPanel(){
	
	     rulesText=new JTextArea(8,20);
	     rulesText.setEditable(true);
	     add(new JLabel("Rules:"));
	     rulesScrollPane=new JScrollPane(rulesText);
	     add(rulesScrollPane);
	     
	     JPanel mutatePanel=new JPanel(new GridLayout(2,1));
	     mutateb=new JButton("Mutate");
             mutateb.setToolTipText("Randomly changes the rules");
             mutateb.addActionListener(this);
             mutatePanel.add(mutateb);
             
             
             undoMutateb=new JButton("Undo");
             undoMutateb.setEnabled(false);
             undoMutateb.setToolTipText("Undo the last 'successful' mutation");
             undoMutateb.addActionListener(this);
             mutatePanel.add(undoMutateb);
             
             add(mutatePanel);
	}
	String getRulesText(){
		return rulesText.getText();
	}
	void rulesToText(Vector rulesVector){
	     String rulesString=""; 
	     
	     for(int i=0;i<rulesVector.size();i++){
	                 
                	rulesString+=((Rule)rulesVector.elementAt(i)).toString() ;
                	rulesString+="\n";
             }     
             rulesText.setText(rulesString);
	
	}
	void setText(String rtxt){
	     rulesText.setText(rtxt);
	     rulesText.setCaretPosition(0);
	}
	//Vector textToRuleVector(){
	//     return (Vector)null;
	//}
	Vector textToRules(){
	        Vector retRulesVector = new Vector();
		Rule.reinit();
		

                String rulesString=rulesText.getText();
                //System.out.println(rulesString);
                StringBuffer onerule;
                int L=rulesString.length();
                int i=0;
                boolean incomment=false;
                while(i<L){
                   onerule=new StringBuffer();
                   
                   while(i<L){
                        char c=rulesString.charAt(i);
                        i++;
                        if(c=='#'){
                           incomment=true;
                        }
                        if(c=='\n'||c=='\r'){
                           
                           incomment=false;
                           break;
                        }
                        if(!incomment && !Character.isWhitespace(c)){
		    	    onerule.append(c);
		    	    
		    	}
		   }
		   if(onerule.length()>0){
		      String onerulestring=onerule.toString();
		      if(Rule.checkRuleString(onerulestring))
                         retRulesVector.add(Rule.stringToRule(onerulestring));
                   }
                }
                
		return retRulesVector;
	}
	public void actionPerformed(ActionEvent e) {
		Object esource=e.getSource();
	        if(esource==(Object)mutateb){
	              Vector rulesVector=textToRules();

	              rulesVector=mutator.do_mutation(rulesVector,0,1);
	              rulesToText(rulesVector);
	              if(mutator.gotUndo)
	                  undoMutateb.setEnabled(true);
	        }
	        if(esource==(Object)undoMutateb){
	              Vector rulesVector=textToRules();

	              rulesVector=mutator.getUndo(rulesVector);
	              rulesToText(rulesVector);
	              undoMutateb.setEnabled(false);
	        }
        
        } 
        void disableMutateUndo(){
        	undoMutateb.setEnabled(false);
        }

}


class ProductionFrame extends JInternalFrame implements ActionListener, Runnable{


        private JTextArea axiomText;
        private JTextArea productionText;
        private JComboBox declareBox;
        private JTextField contextToIgnoreText;
        private String contextToIgnore="[]<>";
        
        private JButton newb,nextb,intbut,initb;
        private IntInputField recursionInput;
        private JLabel recursionsDoneLabel;
        private JLabel pLengthLabel;
        private RulesPanel rulesPanel;
        private int lastdeclareselected=0;
        private int recurse=1; 
        private int numrecursionsdone=0;
              
        private Production p=null;
        private Thread pthread;
        private String inString="",outString="";
        private int outLength;
        
   	public ProductionFrame(){
		super ("Production",true,false,true,true);//JInternalFrame();
		//getContentPane().setLayout(new BorderLayout());
		getContentPane().setLayout(new GridLayout(2,1));
		JPanel  productionPanel,
                        axiomPanel,
                        axiomrulesPanel;
                JScrollPane axiomScrollPane;
		JScrollPane productionScrollPane;

		axiomPanel=new JPanel();
		axiomPanel.add(new JLabel("Axiom:"));
		axiomText= new JTextArea("",1,20);
	
		axiomScrollPane=new JScrollPane(axiomText);
                axiomPanel.add(axiomScrollPane); 
	        //axiomPanel.setToolTipText("Editable");
	        
	        JPanel declarePanel=new JPanel();
	        declarePanel.add(new JLabel("Declare:"));
	        
	        // The declareBox really should use a ComboBoxModel, but doesn't yet
	        declareBox=new JComboBox();
		fillDeclareBox();
	        declareBox.setEditable(true);
	        declarePanel.add(declareBox);
	        declareBox.addActionListener(this);
	        
	        lastdeclareselected=declareBox.getItemCount()-1;

	        
	        JPanel contextToIgnorePanel=new JPanel();
	        contextToIgnorePanel.add(new JLabel("Ignore context"));
	        
	        contextToIgnoreText=new JTextField(contextToIgnore,8);
	        contextToIgnorePanel.add(contextToIgnoreText);
	        //contextToIgnorePanel.setToolTipText("Editable");
	        
	        rulesPanel=new RulesPanel();	
		//rulesPanel.setToolTipText("Editable");
		
		productionPanel=new JPanel(new BorderLayout());
		
		JPanel recursionPanel=new JPanel();
		recursionInput=new IntInputField("Recursions",1,3);
                recursionInput.setMinMax(1,100);
		recursionsDoneLabel=new JLabel("Done: 0");
		pLengthLabel=new JLabel("Length: 0");
		recursionPanel.add (recursionInput);
		recursionPanel.add (recursionsDoneLabel);
		recursionPanel.add (pLengthLabel);
		//recursionPanel.setToolTipText("Editable");
		
		productionText= new JTextArea("",2,40);
		
		productionText.setLineWrap(true);
		productionScrollPane=new JScrollPane(productionText);
		productionPanel.add(recursionPanel,BorderLayout.NORTH);
		productionPanel.add(new JLabel("Production String:"),BorderLayout.WEST);
                productionPanel.add(productionScrollPane,BorderLayout.CENTER); 
                //productionPanel.setToolTipText("Editable");
                
                JPanel pPanelButtonPanel=new JPanel(new GridLayout(2,2));
                newb=new JButton("Make");  
                newb.setToolTipText("Do recursions. Axiom to production string");
                newb.addActionListener(this);                
                pPanelButtonPanel.add(newb);
                
                initb=new JButton("Init");                
                initb.addActionListener(this); 
                initb.setToolTipText("Reset the production");
                pPanelButtonPanel.add(initb);
                
                nextb=new JButton("Next (1)");                
                nextb.addActionListener(this);
                nextb.setToolTipText("Do one more recursion");
                pPanelButtonPanel.add(nextb);
                
                intbut=new JButton("Interrupt");                
                intbut.addActionListener(this);
                intbut.setToolTipText("(Try to) Stop. Enough!");
                pPanelButtonPanel.add(intbut);
                
                productionPanel.add(pPanelButtonPanel,BorderLayout.EAST); 
                productionText.setEditable(true);
                axiomText.setEditable(true);
                
                JPanel axiomdeclarePanel=new JPanel(new GridLayout(3,1));
                axiomdeclarePanel.add(axiomPanel);
                axiomdeclarePanel.add(declarePanel);
                axiomdeclarePanel.add(contextToIgnorePanel);
                axiomrulesPanel=new JPanel();
                axiomrulesPanel.add(axiomdeclarePanel);
                axiomrulesPanel.add(rulesPanel);                

                getContentPane().add(axiomrulesPanel);

                getContentPane().add(productionPanel);
                
                inString=LTools.removeSpaces(axiomText.getText());
                
                pack();
        
	}
	public ProductionFrame(String A, String O, String R){
		this();
		setText(A,O);
		setRules(R);
		inString=LTools.removeSpaces(axiomText.getText());
	}
	String getRulesText(){
		return rulesPanel.getRulesText();
	}
	String getAxiomText(){
		return axiomText.getText();
	}
	void disableMutateUndo(){
		rulesPanel.disableMutateUndo();
	}
	void fillDeclareBox(){
	                	                
	        int cnt1=declareBox.getItemCount();
	        Vector vlist=LTools.expressionParser.getVList();
	        int si=declareBox.getSelectedIndex();
	        declareBox.removeAllItems();
	        int vsz=vlist.size();
	        	
	        for (int i=0;i<vsz;i++){
	        	declareBox.addItem(vlist.elementAt(i));
	        }
	        int cnt2=declareBox.getItemCount();
	        if(cnt2==0) return;
	        if(cnt1>cnt2){//apparently we removed an item
	             if(cnt2>0){
	               declareBox.setSelectedIndex(0);
	             }
	        }
	        else if(cnt2>cnt1){//added an item
	            declareBox.setSelectedIndex(vsz-1);
	        	    
	        }
	        else if(si>=0){//number of items unchanged
	        	     
	            declareBox.setSelectedIndex(si);
	        	    
	        }
	        else if(cnt2>=0){
	            
	            declareBox.setSelectedIndex(lastdeclareselected);
	        }  	

	}
	class PWatcher implements Runnable{
	// a thread for doing low priority updates during a
	// production run
		Thread wthread;
		public PWatcher(){
			wthread=new Thread(this);
			wthread.setPriority(Thread.MIN_PRIORITY);
		}
		public void run(){
		    int n=numrecursionsdone;
		    while(p.interruptme==false){
			if(p.numdone>n){
				n=p.numdone;
				recursionsDoneLabel.setText("Done: "+p.numdone);
				pLengthLabel.setText("Length: "+p.currentlength());
			}
				
			Thread.yield();
		    }
		}
	}
	public void run(){
		String outString="";
	        	        
	        rulesPanel.rulesToText(rulesPanel.textToRules());
	                        
                p=new Production(inString,recurse,contextToIgnoreText.getText());
                
		nextb.setEnabled(false);
		initb.setEnabled(false);
		newb.setEnabled(false);	
		recursionsDoneLabel.setText("Done: "+numrecursionsdone);
		

		PWatcher pw=new PWatcher();
		pw.wthread.start();
                numrecursionsdone+=p.run(); 
                         // (p is not a new thread.) When
                         // p.run returns, we should
                         // have a production  
                p.interruptme=true;//to make sure that the watcher thread quits too
                recursionsDoneLabel.setText("Done: "+numrecursionsdone);                       
		nextb.setEnabled(true);
		initb.setEnabled(true);
		newb.setEnabled(true);
		
                outString=p.getProduction();
                outLength=outString.length();
                pLengthLabel.setText("Length: "+outLength);
                productionText.setText(outString);
                inString=outString;	
	}

	public void actionPerformed(ActionEvent e) {
	        Object esource=e.getSource();
	        if(esource==declareBox){
	                String ds=(String)declareBox.getSelectedItem();
	                if(declareBox.getSelectedIndex()>=0){
	                	lastdeclareselected=declareBox.getSelectedIndex();
				
	                }
			
	                if(ds!=null){
	        	   LTools.expressionParser.getValu(ds);
	        	
	        	   fillDeclareBox();
	        	   
	        	}
	        }
	        if(esource==intbut){
	             if(p!=null)p.interrupt();
	             
	        }
	        if(esource==(Object) newb){ 
	            inString=LTools.removeSpaces(axiomText.getText());
	            axiomText.setText(inString);
	            outString="";
                    outLength=outString.length();
	            productionText.setText(outString);
	            pLengthLabel.setText("Length: 0");
	        }
	        if(esource==(Object)newb || esource==(Object)nextb){
	            if(pthread!=null){
	               //only one at a time should run 
	               if(pthread.isAlive()) return;
	            }
	            if(esource==newb){
	                recurse=recursionInput.getValue();
	                numrecursionsdone=0;
	            }
	            else{
	                recurse=1;
	                inString=LTools.removeSpaces(productionText.getText());
	                if(inString.length()==0){
	                	inString=LTools.removeSpaces(axiomText.getText());
	                }
	            }
	            
        	    pthread=new Thread(this);
        	    pthread.start();
		}        
		if(esource==(Object)initb){
		    outString="";
                    outLength=outString.length();
                    productionText.setText(outString);
                    inString=LTools.removeSpaces(axiomText.getText());
                    axiomText.setText(inString);
                    numrecursionsdone=0;
                    recursionsDoneLabel.setText("Done: "+numrecursionsdone);
                    pLengthLabel.setText("Length: 0"); 
		}

        }
        void reInit(){
                    outString="";
                    outLength=outString.length();
                    pLengthLabel.setText("Length: 0");
                    productionText.setText(outString);
                    numrecursionsdone=0;
                    recursionsDoneLabel.setText("Done: "+numrecursionsdone);
                    inString=LTools.removeSpaces(axiomText.getText());
                    axiomText.setText(inString);
        }
        void setAxiomText(String A){
        	axiomText.setText(A);
        }
	void setText(String A, String O){
	        axiomText.setText(A);
	        productionText.setText(O);
	}
	void setRules(String r){
		rulesPanel.setText(r);
	}
	void setNumRecursions(int r){
		recursionInput.setValue(r);
		recurse=r;
	}
	int getNumRecursions(){
		return recursionInput.getValue();
	}
	String getProduction(){
		return productionText.getText();
	}
	
}
